;  PROG29M - Working through the Emulator  
;
;  Start with the Serial Interface.
;
;  
;  PROG29M - Set Breakpoints
;
;  PROG29L - Get Single Stepping Working Fully
;          - Added Breakpoint Note ("*") on Disassembler 
;
;  PROG29K - Execute a Program
;          - Do a Single Step
;
;  PROG29J - Add the New Program Counter Value Instruction
;          - Put in the "Reset" Code
;
;  PROG29I - Move the Instruction Display to a Subroutine
;
;  PROG39H - Display the Entire Instruction
;
;  PROG39G - Display the Instruction mnemonic
;
;  Prog29F - Get "DownLoad" Working.
; 
;  PROG29E - Display Scratchpad/SFR Data Registers
;
;  PROG29D - Start Working with:
;   U - Unassemble (Display Control Store)
;   C - Change Control Store
;   R - Display Scratchpad RAM and SFR
;   W - Write to Scratchpad RAM and SFR
;
;  PROG29C - Start Doing the Data Interface
;
;  PROG29B - Start with the Command Line Interface.  Upon Return, R0 Points to the
;   First Parameter and R1 Points to the Second.  If there is a Third, Error
;
;   Input String Buffer is 10 Long Starting at 080h.
;
;   Commands are:
;    H        - Help, Display this List
;    T        - Reset Microcontroller
;    D        - Download a New Application
;    E [Addr] - Start Running at the Optional Address
;    1 [Addr] - Single Step from the Optional Address
;             - If just a Carriage Return Received then Single Step 
;    U [Addr] - Unassemble at the PC or Optional Address
;    C Addr   - Modify the Control Store at the Address
;    W Addr   - Write to the Scratchpad RAM/SFR Address
;    R        - Display the Scratchpad RAM Registers
;    B Addr   - Toggle Set Breakpoint at the Specified Address
;    L        - List the Breakpoints
;  
;  PROG29A - Develop Serial Interface - Bounce Back Characters from the Host
;   Terminal.  
;
;  The Serial Interface consists of a Command and a Parameter Followed by a
;   Carriage Return, We Have to Interpret this as well as:
;
;   1.  Ignore Following "Line Feed" - Wait 10 msecs after CR.
;   
;   2.  Handle BackSpacing
;
;  Myke Predko
;  98.06.30
;
;  Hardware Notes:
;  DS87C520 is used as the Microcontroller
;   - Oscillator Speed is 20 MHz
;  P2.7 is Console Serial Output
;  P2.6 is Console Serial Input

;  Constant Declarations
define SerOut=P2.7              ;  Serial Output Bit
define SerIn=P2.6               ;  Serial Input Bit
;define SerOut=P1.0              ;  Serial Output Bit
;define SerIn=P1.6               ;  Serial Input Bit

CR        EQU 00Dh              ;  ASCII Carriage Return
LF        EQU 00Ah              ;  ASCII Line Feed
BS        EQU 008h              ;  ASCII BackSpace

CRLF      EQU 005h              ;  Send CR/LF to the Console

;  Variable Declarations
SavedRegs EQU 080h              ;  Saved Bank Registers and SP, DPTR, IE, PSW, ACC, B
SavedSP   EQU 088h
SavedDPTR EQU 089h
SavedIE   EQU 08Bh
SavedPSW  EQU 08Ch
SavedACC  EQU 08Dh
SavedB    EQU 08Eh

Buffer    EQU 090h              ;  Input/Breakpoint Buffer Area
                                ;   - 10 Characters Long
ProgC     EQU 09Eh              ;  Program Counter for Emulated Device

Step0     EQU 0A0h              ;  Saved Memory for Step Code Saves
Step1     EQU 0B0h

Break0    EQU 0C0h              ;  Saved Memory for Breakpoint Saves
Break1    EQU 0D0h
Break2    EQU 0E0h

;  Saved Memory for Step Code Structure:
;  Byte 0   - 0/0FFh - "0FFh" Indicates that the Code Must be Replaced
;  Byte 1/2 - Address of the Breakpoint/Step
;  Byte 3+  - Program code that was replaced with the Breakpoint  

MonitorSP EQU 0EFh              ;  Put the Monitor Stack at the Top of Scratchpad RAM 


;  Macros


;  Mainline
 org 0                          ;  Execution Starts Here

  ljmp    Mainline

 org 003h                       ;  Int0 Pin Interrupt

  ajmp    _Int0

 org 00Bh                       ;  Tmr0 Interrupt

  ajmp    _Tmr0

 org 013h                       ;  Int1 Pin Interrupt

  ajmp    _Int1

 org 01Bh                       ;  Tmr1 Interrupt

  ajmp    _Tmr1

 org 023h                       ;  Serial Port Interrupt

_Ser:

  mov     TH2,A
  mov     A,TL2                 ;  Get the Operation Type
  cjne    A,#1,_Ser_Step        ;  Is it a Step or Execute?  
                                ;   Execute
  mov     A,TH2                 ;  Restore ACC
  ljmp    04023h                ;  - Execute the Interrupt Handler

_Ser_Step:                      ;  Now, Return to the Step Location

  mov     A,#023h               ;  Put the Interrupt Handler Address on the Stack
  push    ACC
  mov     A,#040h
  push    ACC

  ljmp    Step_Done

_Int0:

  mov     TH2,A
  mov     A,TL2                 ;  Get the Operation Type
  cjne    A,#1,_Int0_Step       ;  Is it a Step or Execute?  
                                ;   Execute
  mov     A,TH2                 ;  Restore ACC
  ljmp    04003h                ;  - Execute the Interrupt Handler

_Int0_Step:                     ;  Now, Return to the Step Location

  mov     A,#003h               ;  Put the Interrupt Handler Address on the Stack
  push    ACC
  mov     A,#040h
  push    ACC

  ljmp    Step_Done

_Int1:

  mov     TH2,A
  mov     A,TL2                 ;  Get the Operation Type
  cjne    A,#1,_Int1_Step       ;  Is it a Step or Execute?  
                                ;   Execute
  mov     A,TH2                 ;  Restore ACC
  ljmp    04013h                ;  - Execute the Interrupt Handler

_Int1_Step:                     ;  Now, Return to the Step Location

  mov     A,#013h               ;  Put the Interrupt Handler Address on the Stack
  push    ACC
  mov     A,#040h
  push    ACC

  ljmp    Step_Done

_Tmr1:

  mov     TH2,A
  mov     A,TL2                 ;  Get the Operation Type
  cjne    A,#1,_Tmr1_Step       ;  Is it a Step or Execute?  
                                ;   Execute
  mov     A,TH2                 ;  Restore ACC
  ljmp    0401Bh                ;  - Execute the Interrupt Handler

_Tmr1_Step:                     ;  Now, Return to the Step Location

  mov     A,#01Bh               ;  Put the Interrupt Handler Address on the Stack
  push    ACC
  mov     A,#040h
  push    ACC

  ljmp    Step_Done

_Tmr0:

  mov     TH2,A
  mov     A,TL2                 ;  Get the Operation Type
  cjne    A,#1,_Tmr0_Step       ;  Is it a Step or Execute?  
                                ;   Execute
  mov     A,TH2                 ;  Restore ACC
  ljmp    0400Bh                ;  - Execute the Interrupt Handler

_Tmr0_Step:                     ;  Now, Return to the Step Location

  mov     A,#00Bh               ;  Put the Interrupt Handler Address on the Stack
  push    ACC
  mov     A,#040h
  push    ACC

  ljmp    Step_Done

MainLine:                       ;  Mainline of the Program

  mov     R0,#Break0            ;  Make Sure all the Breakpoints Turned Off
  mov     @R0,#0
  mov     R0,#Break1
  mov     @R0,#0
  mov     R0,#Break2
  mov     @R0,#0

  mov     SP,#MonitorSP         ;  Use my Stack Pointer, Rather than App's

  lcall   EMUReset              ;  Make Sure the Emulated Part is Reset

  mov     A,#0                  ;  Display the Introductory Message
  lcall   SendMsg

  lcall   DisplayHelp1          ;  As the First Thing, Display Help
  lcall   DisplayHelp2
  lcall   DisplayHelp3

Loop:

  lcall   Prompt                ;  Set the Prompt

  lcall   GetCommand            ;  Wait for the Command to Come in

  mov     A,R0
  xrl     A,#0FFh
  jz      Step                  ;  If Nothing Entered, Step
  ljmp    Not_Step
Step:                           ;  If Nothing Entered, Then Step

  mov     R0,#ProgC             ;  Get the Program Counter
  mov     DPL,@R0               ;  Set up the Pointer to the Next Address
  mov     DPL1,@R0              ;  Save for the Execute
  inc     R0
  mov     A,@R0
  mov     DPH1,A                ;  Make Sure Useable Address in DPTR1
  add     A,#040h
  mov     DPH,A

  movx A,@DPTR                  ;  Load the Current Instruction
  mov     R4,A                  ;  Save the Instruction

  mov     R0,#Step1
  mov     @R0,#0                ;  Mark that Step 1 is Not Used

  lcall   InsType2              ;  Find Out What Type of Instruction it is
  clr     C
  subb    A,#28                 ;  Is it Not a Jump?
  jnc     Step_SkipA
  ajmp    Step_NoJump
Step_SkipA:

  mov     A,R4
  lcall   InsType2
  clr     C
  subb    A,#39
  jc      Step_SkipB
  ajmp    Step_NoJump
Step_SkipB:

  mov     TL2,DPL               ;  Save the Instruction Address
  mov     TH2,DPH

  mov     A,R4                  ;  Figure Out Where the Instruction is Going
  lcall   InsType2
  clr     C
  subb    A,#28                 ;  Do we Have an 11 Bit Jump?
  jnz     Step_Tryljmp

  mov     A,R4                  ;  Yes...  Make an 11 Bit Address
  swap A
  rr A
  anl     A,#7
  add     A,#040h
  mov     R3,A

  inc     DPTR
  movx A,@DPTR
  mov     R2,A

  sjmp    Step_jmpDone

Step_Tryljmp:

  dec     A                     ;  Do we Have Instruction Type 29?
  jnz     Step_BitCheck

  inc     DPTR                  ;  Yes, Get it's address
  movx A,@DPTR
  mov     R2,A
  inc     DPTR
  movx A,@DPTR
  add     A,#040h
  mov     R3,A

  sjmp    Step_jmpDone

Step_BitCheck:                  ;  Do we Have a Bit Operation?  

  dec     A
  jnz     Step_jmpCheck         ;  See if we have a Bit or Jmp Operation
  
  sjmp    Step_DoBit

Step_jmpCheck:

  xrl     A,#8                  ;  '38' is the only other option
  jnz     Step_Bit

  mov     R0,#SavedACC          ;  '38' "jmp @A+DPTR"
  mov     A,@R0
  mov     R0,#SavedDPTR
  add     A,@R0
  mov     R2,A
  inc     R0
  mov     A,@R0
  addc    A,#040h
  mov     R3,A

  sjmp    Step_jmpDone

Step_Bit:

  xrl     A,#00Eh               ;  Do we Have 36?
  jz      Step_DoBit            ;   If Yes, it's only 2 Bytes Long

  inc     DPTR                  ;  Point to the Instruction Past

Step_DoBit:

  inc     DPTR

  movx A,@DPTR                  ;  Get the Offset

  jb      ACC.7,Step_BitNeg     ;  Do the Negative Subtract

  add     A,TL2
  mov     R2,A
  mov     A,TH2
  addc    A,#0
  mov     R3,A

  sjmp    Step_jmpDone

Step_BitNeg:                    ;  Negative, Take the Value Away from the Current

  mov     R2,A
  mov     A,#0
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,TL2
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,TH2
  subb    A,#0
  mov     R3,A

Step_jmpDone

  mov     DPL,TL2               ;  Restore the Current Address
  mov     DPH,TH2

  mov     A,DPL                 ;  Now, is the Jump Before or After?  
  clr     C
  subb    A,R2
  mov     R4,A
  mov     A,DPH
  subb    A,R3
  mov     R5,A

  jnb     ACC.7,Step_Checkjmp   ;  If Negative, Make Positive

  mov     A,#0                  ;  Subtract the Difference from Zero
  clr     C
  subb    A,R4
  mov     R4,A
  mov     A,#0
  subb    A,R5
  mov     R5,A

Step_Checkjmp:                  ;  If Difference is < 8, then Can't Step

  cjne    R5,#0,Step_Dojmp      ;  If R3 != 0, then Jump is Okay

  mov     A,R4                  ;  If at least 9 Away, no Problems
  clr     C
  subb    A,#9
  jnc     Step_Dojmp

  mov     A,#6
  lcall   SendMsg2

  ljmp    Loop

Step_Dojmp:                     ;  Can do the Jump

  mov     R0,#Step1             ;  Have to Save the Second Jump
  mov     @R0,#0FFh             ;  Mark that we have a Step1
  inc     R0

  mov     @R0,2                 ;  Save the Address to Change
  inc     R0
  mov     @R0,3
  inc     R0

  mov     DPL,R2
  mov     DPH,R3

  mov     R1,#(Step_Done - Step_Proto)  ;   with "Step_Proto
  mov     R2,#(Step_Proto - Step1_Get)

Step_Loop1:                     ;  Loop Here Until Step Saved/New Code Put in

  movx A,@DPTR                  ;  Get the Next Byte
  mov     @R0,A                 ;  Save it in "Step0"

  mov     A,R2                  ;  Get the Next Byte to Change
  movc A,@A+PC
Step1_Get:
  movx @DPTR,A                  ;   Save it into External Memory

  inc     DPTR                  ;  Point to the Next Addresses
  inc     R0
  inc     R2

  djnz    R1,Step_Loop1         ;  Loop Around Again?  

  mov     DPL,TL2               ;  Restore the Address
  mov     DPH,TH2

Step_NoJump:                    ;  Now, Do the First Stop

;  #### - What if "ret"/"reti" is encountered, Have to Place a Stop Point at the 
;          Return Address

  movx A,@DPTR                  ;  Get the Current Character Again
  mov     A,R4
  lcall   InsLength             ;  Figure Out How Many Bytes Long the Instruction Is
  
  add     A,DPL                 ;  Point to the Next Instruction
  mov     DPL,A
  mov     A,DPH
  addc    A,#0                  ;  Make Sure Carry is Added
  mov     DPH,A

  mov     R0,#Step0             ;  Now, Save the Next few Bytes and Load them
  mov     R1,#(Step_Done - Step_Proto)  ;   with "Step_Proto
  mov     R2,#(Step_Proto - Step_Get)

  mov     @R0,#0FFh             ;  Mark that Step0 was Used
  inc     R0

  mov     @R0,DPL               ;  Save the Address that was updated
  inc     R0
  mov     @R0,DPH
  inc     R0

Step_Loop0:                     ;  Loop Here Until Step Saved/New Code Put in

  movx A,@DPTR                  ;  Get the Next Byte
  mov     @R0,A                 ;  Save it in "Step0"

  mov     A,R2                  ;  Get the Next Byte to Change
  movc A,@A+PC
Step_Get:
  movx @DPTR,A                  ;   Save it into External Memory

  inc     DPTR                  ;  Point to the Next Addresses
  inc     R0
  inc     R2

  djnz    R1,Step_Loop0         ;  Loop Around Again?
  
  mov     TL2,#0                ;  Mark that We are Stepping

  ajmp    ProgExecute           ;  Execute the Program (One Instruction)
  
Step_Proto:                     ;  Code Loaded After Current Instruction

  jbc     IE.7,Step_Skip
  inc     DPL1                  ;  Mark that Interrupts were Enabled
Step_Skip:
  lcall   Step_Done             ;  Finished, Load the Code

Step_Done:                      ;  Return Here after a Step/Breakpoint
                                ;   Save the Context Registers

  mov     DPH1,R0               ;  Save R0 so it can be used as an Index
  
  mov     R0,#SavedRegs         ;  Now, Save the Registers

  mov     @R0,DPH1              ;  Save R0
  inc     R0
  mov     @R0,1
  inc     R0
  mov     @R0,2
  inc     R0
  mov     @R0,3
  inc     R0
  mov     @R0,4
  inc     R0
  mov     @R0,5
  inc     R0
  mov     @R0,6
  inc     R0
  mov     @R0,7
  inc     R0

  mov     R2,A                  ;  Save ACC for Updating Later
  mov     R5,PSW                ;  Save PSW for Updating Later
  
  mov     R1,SP                 ;  Figure Out what was the Last Address

  mov     SP,#MonitorSP

SD_StackCheck:                  ;  Look for a Valid Address to Save

  mov     A,@R1                 ;  Get the High Byte
  jb      ACC.6,SD_StackEnd     ;   If Bit 6 is Set, then not in Control Store

  dec     R1
  dec     R1                    ;  Else, Try the Next Stack Entry

  ajmp    SD_StackCheck

SD_StackEnd:                    ;  Have the Return Address at R1

  dec     R1                    ;  Decrement the Stack Pointer to Actual Value
  dec     R1
  mov     @R0,1                 ;  Save the Correct Stack Pointer
  inc     R1                    ;  Restore the Stack Pointer to Get the Address
  inc     R1
  inc     R0
  mov     @R0,DPL               ;  Save the DPTR
  inc     R0
  mov     @R0,DPH
  inc     R0
  mov     A,DPL1                ;  Now Figure out the EA bit
  mov     C,ACC.0               ;  Save the 
  mov     A,IE                  ;  Save the Interupt Handler
  mov     ACC.7,C               ;  Transfer the Interrupt Enable Bit
  mov     @R0,A                 ;   Save the Application's IE Register
  mov     IE,#0                 ;  Clear the Interrupt Handler
  inc     R0
  mov     @R0,5                 ;  Save the Application's PSW
  inc     R0
  mov     @R0,2                 ;  Save the Application's ACC
  inc     R0
  mov     @R0,B                 ;  Save the Application's "B" Register

  mov     R0,#ProgC             ;  Point to the Top of the Program Counter 
  mov     A,@R1                 ;  Get the High Byte of the Application's Program Counter
  clr     C
  subb    A,#040h
  mov     R3,A
  dec     R1
  mov     A,@R1                 ;  Save the Low Byte of the PC
  subb    A,#(Step_Done - Step_Proto)
  mov     @R0,A
  inc     R0                    ;  Now, Save the high Byte
  mov     A,R3
  subb    A,#0                  ;  Get the Carry Bit (If Appropriate)
  mov     @R0,A

  mov     A,TL2                 ;  Now, Do We Clear Steps or Breakpoints?
  cjne    A,#0,Clear_BP

Clear_Step:                     ;  Clear the Step Stuff

  mov     R0,#Step0             ;  Do we Have a Step to Replace Code On?
  cjne    @R0,#0FFh,Not_Step0   ;  If Not Equal To Zero, Check the Other Step

  inc     R0
  mov     DPL,@R0               ;  Point to the Location Where the Step Took Place
  inc     R0
  mov     DPH,@R0

  mov     R1,#(Step_Done - Step_Proto)  ;   Number of Bytes to Restore

DoStep_Loop:                    ;  copy the Data Back in

  inc     R0

  mov     A,@R0
  movx @DPTR,A

  inc     DPTR

  djnz    R1,DoStep_Loop

Not_Step0:

  mov     R0,#Step1             ;  Do we Have a Step to Replace Code On?
  cjne    @R0,#0FFh,Clear_Done  ;  If Not Equal To Zero, Check the Other Step

  inc     R0
  mov     DPL,@R0               ;  Point to the Location Where the Step Took Place
  inc     R0
  mov     DPH,@R0

  mov     R1,#(Step_Done - Step_Proto)  ;   Number of Bytes to Restore

DoStep1_Loop:                   ;  copy the Data Back in

  inc     R0

  mov     A,@R0
  movx @DPTR,A

  inc     DPTR

  djnz    R1,DoStep1_Loop

  sjmp    Clear_Done

Clear_BP:

  mov     R0,#Break0            ;  Do we Have a Step to Replace Code On?
  cjne    @R0,#0FFh,Not_Break0  ;  If Not Equal To Zero, Check the Other Step

  inc     R0
  mov     DPL,@R0               ;  Point to the Location Where the Step Took Place
  inc     R0
  mov     DPH,@R0

  mov     R1,#(Step_Done - Step_Proto)  ;   Number of Bytes to Restore

DoBreak0_Loop:                  ;  copy the Data Back in

  inc     R0

  mov     A,@R0
  movx @DPTR,A

  inc     DPTR

  djnz    R1,DoBreak0_Loop

Not_Break0:

  mov     R0,#Break1            ;  Do we Have a Step to Replace Code On?
  cjne    @R0,#0FFh,Not_Break1  ;  If Not Equal To Zero, Check the Other Step

  inc     R0
  mov     DPL,@R0               ;  Point to the Location Where the Step Took Place
  inc     R0
  mov     DPH,@R0

  mov     R1,#(Step_Done - Step_Proto)  ;   Number of Bytes to Restore

DoBreak1_Loop:                  ;  copy the Data Back in

  inc     R0

  mov     A,@R0
  movx @DPTR,A

  inc     DPTR

  djnz    R1,DoBreak1_Loop

Not_Break1:

  mov     R0,#Break2            ;  Do we Have a Step to Replace Code On?
  cjne    @R0,#0FFh,Not_Break2  ;  If Not Equal To Zero, Check the Other Step

  inc     R0
  mov     DPL,@R0               ;  Point to the Location Where the Step Took Place
  inc     R0
  mov     DPH,@R0

  mov     R1,#(Step_Done - Step_Proto)  ;   Number of Bytes to Restore

DoBreak2_Loop:                  ;  copy the Data Back in

  inc     R0

  mov     A,@R0
  movx @DPTR,A

  inc     DPTR

  djnz    R1,DoBreak2_Loop

Not_Break2:
Clear_Done:

  ljmp    Loop

Not_Step:                       ;  Look at the Different Commands

  mov     A,@R0                 ;  Get the Command
  xrl     A,#'T'                ;  Reset the Processor?
  jnz     Not_Reset

  lcall   EMUReset              ;  Reset the Emulated Part

  ljmp    Loop

Not_Reset:

  mov     A,@R0                 ;  Change the Program Counter?
  xrl     A,#'P'
  jz      NewPC
  ajmp    No_NewPC
NewPC:

  cjne    R1,#0FFh,PC_New       ;  Is there a Valid Address for the PC?

  mov     A,#4                  ;  No Address Specified
  lcall   SendMsg

  ljmp    Loop

PC_New:

  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

PC_Loop:                        ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 Bits for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,PCCHex_Skip1    ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
PCCHex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     PC_Loop               ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      PCAddr_OK

  mov     A,#3                  ;  Put in the Invalid Address Error
  lcall   SendMsg

  ljmp    Loop

PCAddr_OK:                      ;  The Address is OK - Now Get the New Value

  mov     R0,#ProgC             ;  Save the New Program Counter Address

  mov     @R0,DPL               ;  Low Byte First
  inc     R0
  mov     @R0,DPH

  ljmp    Loop

No_NewPC:

  mov     A,@R0                 ;  Download a New Application?
  xrl     A,#'D'
  jz      Download
  ajmp    Not_DownLoad
Download:

  mov     A,#0                  ;  Prompt the User to Start the Download
  lcall   SendMsg2

  mov     R0,#070h              ;  Wait for 60 Seconds for the First Character
  mov     R1,#017h

DL_First:                       ;  Wait for the First Character

  lcall   GetChar               ;  Poll for an Incoming Character

  jnc     DL_1stCheck           ;  Character, Check to See if Something Coming In

  djnz    R0,DL_First           ;  Decrement the Low Byte
  djnz    R1,DL_First

Download_TO:                    ;  Timeout Message

  mov     A,#1                  ;  Put in a "Timeout" Message
  lcall   SendMsg2

  ljmp    Loop

Download_Loop:                  ;  Loop Here for Each Line

  lcall   GetChar               ;  Anything Coming through?
  
  jc      Download_TO           ;  Timeout - Error

DL_1stCheck:                    ;  Check to see if 1st Character is a Colon

  xrl     A,#':'                ;  Do we Have a Colon?  
  jz      DL_Line               ;   Yes - Download the Line

  mov     A,#2                  ;  Error - Wrong Line Format
  lcall   SendMsg2

  ljmp    Loop

DL_Line:                        ;  Now, Download the Line

  lcall   GetChar               ;  Get the Next Character to Download
  jc      Download_TO      

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip1        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     R5,A                  ;  Use "R5" as the Bit Counter 

  lcall   GetChar               ;  Get the Next Character to Download
  jc      Download_TO      

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip2        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip2:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  swap A
  orl     A,R5                  ;  Add the High Byte
  swap A
  mov     R5,A                  ;  Use "R5" as the Incoming Byte Counter 

  lcall   GetChar               ;  Get the Address to Download
  jc      Download_TO      

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip3        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip3:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     DPH,A                 ;  Use "DPH" as the Final Destination 

  lcall   GetChar               ;  Get the Next Character to Download
  jc      Download_TO      

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip4        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip4:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  swap A
  orl     A,DPH                 ;  Add the High Byte
  swap A
  add     A,#040h               ;  Make into an External Value
  mov     DPH,A  

  lcall   GetChar               ;  Get the Address to Download
  jc      Download_TO      

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip5        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip5:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     DPL,A                 ;  Use "DPL" as the Final Destination 

  lcall   GetChar               ;  Get the Next Character to Download
  jc      Download_TO      

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip6        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip6:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  swap A
  orl     A,DPL                 ;  Add the High Byte
  swap A
  mov     DPL,A
  
  lcall   GetChar               ;  Get the Data/End Character
  jnc     DL_SkipB1
  ajmp    Download_TO
DL_SkipB1:

  lcall   GetChar
  jnc     DL_SkipB2
  ajmp    Download_TO
DL_SkipB2:

  xrl     A,#'1'                ;  Do we Have an End Line?
  jnz     Download_Do           ;   If not == '1', then No, Download the Line

  lcall   GetChar               ;  Have Potentially 5 Characters Following
  lcall   GetChar
  lcall   GetChar
  lcall   GetChar
  lcall   GetChar

  lcall   EMUReset

  mov     R0,#Break0            ;  Make Sure all the Breakpoints Turned Off
  mov     @R0,#0
  mov     R0,#Break1
  mov     @R0,#0
  mov     R0,#Break2
  mov     @R0,#0

  ljmp    Loop

Download_Do:                    ;  Download Each Character

  lcall   GetChar               ;  Get the Byte to Save
  jnc     DL_SkipA1
  ajmp    Download_TO
DL_SkipA1:  

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip7        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip7:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     R4,A                   

  lcall   GetChar               ;  Get the Next Character to Download
  jnc     DL_SkipA2
  ajmp    Download_TO
DL_SkipA2:  

  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,DL_Skip8        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
DL_Skip8:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  swap A
  orl     A,R4                  ;  Add the High Byte
  swap A
  movx @DPTR,A                  ;  Save the Byte

  inc     DPTR                  ;  Point to the Next Address in External Memory

  djnz    R5,Download_Do

  lcall   GetChar               ;  Now, Read Past the Two Checksum Bytes
  jnc     DL_SkipA3
  ajmp    Download_TO
DL_SkipA3:
  lcall   GetChar
  jnc     DL_SkipA4
  ajmp    Download_TO
DL_SkipA4:

  lcall   GetChar               ;  Now, Read Past the CR and Optional LF
  jnc     DL_SkipA5
  ajmp    Download_TO
DL_SkipA5:
  lcall   GetChar
  jnc     DL_SkipA6
  ajmp    Download_TO
DL_SkipA6:

  xrl     A,#':'                ;  Did we End up with a Colon?
  jnz     DL_SkipA7
  ajmp    DL_Line               ;   - Yes, Start from Here
DL_SkipA7:

  ajmp    Download_Loop         ;  No, Loop Around 

Not_Download:                   ;  Execute the Application?

  mov     A,@R0
  xrl     A,#'E'
  jz      Execute
  ajmp    Not_Execute
Execute:

  cjne    R1,#0FFh,GetEAddr     ;  Is there a Valid Address to Write to?

  mov     R0,#ProgC             ;  Read the Program Counter into DPTR1
  mov     A,@R0
  mov     DPL1,A
  inc     R0
  mov     A,@R0
  mov     DPH1,A

  ajmp    HaveEAddr

GetEAddr:                       ;  Get the Address Specified from the Command

  mov     DPL1,#0               ;  Store the Address in DPTR
  mov     DPH1,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

EAddr_Loop:                     ;  Loop Here Until R1 == R2

  mov     A,DPH1                ;  Shift DPTR up by 4 for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH1,A

  mov     A,DPL1                ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH1
  mov     DPH1,A

  mov     A,DPL1                ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL1,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,EHex_Skip1      ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
EHex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL1
  mov     DPL1,A                ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     EAddr_Loop            ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH1                ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      HaveEAddr

  mov     A,#3                  ;  Put in the Invalid Address Error
  lcall   SendMsg

  ljmp    Loop

HaveEAddr:

  mov     R0,#Step0             ;  Mark that Steps Weren't Used
  mov     @R0,#0

  mov     R0,#Step1
  mov     @R0,#0

  mov     TL2,#1                ;  Mark that We are Executing

  mov     R0,#ProgC             ;  Get the Current Program Counter for Checking
  mov     6,@R0
  inc     R0
  mov     7,@R0

ECheck_B0:

  mov     R0,#Break0            ;  See if we Have any Breakpoints to Set
  cjne    @R0,#0FFh,ECheck_B1   ;  If Nothing Set, then Check BP1
  
  inc     R0                    ;  Get the Address
  mov     4,@R0
  inc     R0
  mov     5,@R0

  mov     A,R4                  ;  How Far from Executing Code?
  clr     C
  subb    A,R6
  mov     R4,A
  mov     A,R5
  subb    A,R7
  mov     R7,A

  jnb     ACC.7,ECheck_B1       ;  If Positive, See if the Value is Greater than 8

  mov     A,#0
  clr     C
  subb    A,R4
  mov     R4,A
  mov     A,#0
  subb    A,R5

  jnz     ECheck_B1
  mov     A,R4
  clr     C
  subb    A,#9
  jnc     ECheck_B1

  mov     A,#7                  ;  Cannot Load the Breakpoint
  lcall   SendMsg2

  lcall   Loop

ECheck_B1:

  mov     R0,#Break1            ;  See if we Have any Breakpoints to Set
  cjne    @R0,#0FFh,ECheck_B2   ;  If Nothing Set, then Check BP1
  
  inc     R0                    ;  Get the Address
  mov     4,@R0
  inc     R0
  mov     5,@R0

  mov     A,R4                  ;  How Far from Executing Code?
  clr     C
  subb    A,R6
  mov     R4,A
  mov     A,R5
  subb    A,R7
  mov     R7,A

  jnb     ACC.7,ECheck_B2       ;  If Positive, See if the Value is Greater than 8

  mov     A,#0
  clr     C
  subb    A,R4
  mov     R4,A
  mov     A,#0
  subb    A,R5

  jnz     ECheck_B2
  mov     A,R4
  clr     C
  subb    A,#9
  jnc     ECheck_B2

  mov     A,#7                  ;  Cannot Load the Breakpoint
  lcall   SendMsg2

  lcall   Loop

ECheck_B2:

  mov     R0,#Break2            ;  See if we Have any Breakpoints to Set
  cjne    @R0,#0FFh,EB0_Do      ;  If Nothing Set, then Can Put in Breakpoints
  
  inc     R0                    ;  Get the Address
  mov     4,@R0
  inc     R0
  mov     5,@R0

  mov     A,R4                  ;  How Far from Executing Code?
  clr     C
  subb    A,R6
  mov     R4,A
  mov     A,R5
  subb    A,R7
  mov     R7,A

  jnb     ACC.7,EB0_Do          ;  If Positive, See if the Value is Greater than 8

  mov     A,#0
  clr     C
  subb    A,R4
  mov     R4,A
  mov     A,#0
  subb    A,R5

  jnz     EB0_Do
  mov     A,R4
  clr     C
  subb    A,#9
  jnc     EB0_Do

  mov     A,#7                  ;  Cannot Load the Breakpoint
  lcall   SendMsg2

  lcall   Loop

EB0_Do:                         ;  Put in the Breakpoint Code

  mov     R0,#Break0            ;  See if we Have any Breakpoints to Set
  cjne    @R0,#0FFh,EB1_Do      ;  If Nothing Set, then Check BP1

  inc     R0                    ;  Get the Address to Change
  mov     DPL,@R0
  inc     R0
  mov     A,@R0
  add     A,#040h               ;  Put in External Memory 
  mov     DPH,A
  inc     R0                    ;  Pointing to Data Save Aread

  mov     R1,#(BreakCode_Done - Break_Proto)  ;   with "Step_Proto
  mov     R2,#(Break_Proto - Break0_Get)

Break0_Loop:                    ;  Loop Here Until Step Saved/New Code Put in

  movx A,@DPTR                  ;  Get the Next Byte
  mov     @R0,A                 ;  Save it in "Step0"

  mov     A,R2                  ;  Get the Next Byte to Change
  movc A,@A+PC
Break0_Get:
  movx @DPTR,A                  ;   Save it into External Memory

  inc     DPTR                  ;  Point to the Next Addresses
  inc     R0
  inc     R2

  djnz    R1,Break0_Loop        ;  Loop Around Again?

EB1_Do:                         ;  Put in the Breakpoint Code

  mov     R0,#Break1            ;  See if we Have any Breakpoints to Set
  cjne    @R0,#0FFh,EB2_Do      ;  If Nothing Set, then Check BP1

  inc     R0                    ;  Get the Address to Change
  mov     DPL,@R0
  inc     R0
  mov     A,@R0
  add     A,#040h               ;  Put in External Memory 
  mov     DPH,A
  inc     R0                    ;  Pointing to Data Save Aread

  mov     R1,#(BreakCode_Done - Break_Proto)  ;   with "Step_Proto
  mov     R2,#(Break_Proto - Break1_Get)

Break1_Loop:                    ;  Loop Here Until Step Saved/New Code Put in

  movx A,@DPTR                  ;  Get the Next Byte
  mov     @R0,A                 ;  Save it in "Step0"

  mov     A,R2                  ;  Get the Next Byte to Change
  movc A,@A+PC
Break1_Get:
  movx @DPTR,A                  ;   Save it into External Memory

  inc     DPTR                  ;  Point to the Next Addresses
  inc     R0
  inc     R2

  djnz    R1,Break1_Loop        ;  Loop Around Again?

EB2_Do:                         ;  Put in the Breakpoint Code

  mov     R0,#Break2            ;  See if we Have any Breakpoints to Set
  cjne    @R0,#0FFh,ProgExecute ;  If Nothing Set, then Check BP1

  inc     R0                    ;  Get the Address to Change
  mov     DPL,@R0
  inc     R0
  mov     A,@R0
  add     A,#040h               ;  Put in External Memory 
  mov     DPH,A
  inc     R0                    ;  Pointing to Data Save Aread

  mov     R1,#(BreakCode_Done - Break_Proto)  ;   with "Step_Proto
  mov     R2,#(Break_Proto - Break2_Get)

Break2_Loop:                    ;  Loop Here Until Step Saved/New Code Put in

  movx A,@DPTR                  ;  Get the Next Byte
  mov     @R0,A                 ;  Save it in "Step0"

  mov     A,R2                  ;  Get the Next Byte to Change
  movc A,@A+PC
Break2_Get:
  movx @DPTR,A                  ;   Save it into External Memory

  inc     DPTR                  ;  Point to the Next Addresses
  inc     R0
  inc     R2

  djnz    R1,Break2_Loop        ;  Loop Around Again?

ProgExecute:                    ;  Execute the Program at DPTR1

  mov     R0,#SavedSP           ;  Push the Program Counter onto the Stack
  mov     SP,@R0

  push    DPL1
  mov     A,DPH1                ;  Make Sure you Have the Correct Address
  add     A,#040h
  push    ACC

  mov     DPL1,#0               ;  Now, Use DPL1 for Interrupt Flag

  mov     R0,#SavedB            ;  Now, Restore the Other Registers

  mov     B,@R0                 ;  Restore the "B" Register

  dec     R0                    ;  Restore "ACC"
  mov     A,@R0

  dec     R0                    ;  Restore the "PSW" Register
  mov     PSW,@R0

  dec     R0
  mov     DPL,@R0               ;  Put IE on the Stack without Setting IE
  push    DPL

  dec     R0                    ;  Restore the DPTR
  mov     DPL,@R0
  dec     R0
  mov     DPH,@R0

  dec     R0                    ;  Now, Restore the Bank Registers
  dec     R0
  mov     7,@R0
  dec     R0
  mov     6,@R0
  dec     R0
  mov     5,@R0
  dec     R0
  mov     4,@R0
  dec     R0
  mov     3,@R0
  dec     R0
  mov     2,@R0
  dec     R0
  mov     1,@R0
  dec     R0                    ;  Finally, Replace R0 with the Correct Values   
  mov     0,@R0

  pop     IE
  ret                           ;  Execute from the Stack

  
Break_Proto:                    ;  Code Loaded For Breakpoints

  jbc     IE.7,Break_Skip
  inc     DPL1                  ;  Mark that Interrupts were Enabled
Break_Skip:
  lcall   Step_Done             ;  Finished, Load the Code
BreakCode_Done:                 ;  Finished Address


Not_Execute:                    ;  Single Step?

  mov     A,@R0
  xrl     A,#'1'
  jnz     Step_Over

  cjne    R1,#0FFh,Get1Addr     ;  Is there a Valid Address to Write to?

  mov     R0,#ProgC             ;  Read the Program Counter into DPTR
  mov     A,@R0
  mov     DPL,A
  inc     R0
  mov     A,@R0
  mov     DPH,A

  ajmp    Have1Addr

Get1Addr:                       ;  Get the Address Specified from the Command

  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

Addr1_Loop:                     ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,Hex1_Skip1      ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
Hex1_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     Addr1_Loop            ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      Have1Addr

  mov     A,#3                  ;  Put in the Invalid Address Error
  lcall   SendMsg

  ljmp    Loop

Have1Addr:

  mov     R0,#ProgC             ;  Set "ProgC" with the Correct Stepping Address
  mov     @R0,DPL
  inc     @R0
  mov     @R0,DPH

  ajmp    Step                  ;  Yes, Jump to Single Step Algorithm

Step_Over:

  mov     A,@R0                 ;  Unassemble Some Code?
  xrl     A,#'U'
  jz      Unassemble
  ajmp    Not_Unassemble
Unassemble:

  cjne    R1,#0FFh,GetUAddr     ;  Is there a Valid Address to Write to?

  mov     R0,#ProgC             ;  Read the Program Counter into DPTR
  mov     A,@R0
  mov     DPL,A
  inc     R0
  mov     A,@R0
  mov     DPH,A

  ajmp    HaveUAddr

GetUAddr:                       ;  Get the Address Specified from the Command

  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

UAddr_Loop:                     ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,UHex_Skip1      ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
UHex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     UAddr_Loop            ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      HaveUAddr

  mov     A,#3                  ;  Put in the Invalid Address Error
  lcall   SendMsg

  ljmp    Loop

HaveUAddr:

  mov     A,DPH                 ;  Put the Address in the External RAM
  add     A,#040h               ;   Starts at 16K
  mov     DPH,A

  mov     R5,#23                ;  Display 23 Lines

HUA_Loop:                       ;  Loop For Each Instruction

  movx A,@DPTR                  ;  Read the Value from External Memory
  mov     R4,A                  ;  Save the Instruction for Later

  lcall   InsLength             ;  Get the Instruction Length
  mov     R3,A

  mov     A,DPH                 ;  Display the Address
  clr     C
  subb    A,#040h               ;  Make Sure Address is correct
  lcall   SendHex
  mov     A,DPL
  lcall   SendHex

  mov     A,#7                  ;  After Address, Display ":  " 
  lcall   SendMsg

HUA_Loop1:                      ;  Now, Display Each Instruction 

  mov     A,DPL1                ;  Shift Up the Last Two Bytes
  mov     DPH1,A

  movx A,@DPTR                  ;  Get the Byte to Display
  mov     DPL1,A
  lcall   SendHex

  mov     A,#' '
  lcall   SendChar

  inc     DPTR

  djnz    R3,HUA_Loop1

  mov     A,R4                  ;  Get the First Byte of the Instruction
  lcall   InsLength             ;  Get the Instruction Length

  xrl     A,#0FFh               ;  Make the Length into a Negative Number
  inc     A
  add     A,#3                  ;  See How Many To get a Full 3 Bytes for Instruction
  jz      HUA_Skip2             ;  If Zero, Skip Over it
  mov     R3,A                  ;  Save the Number of Characters to Update

HUA_Loop2:                      ;  Now, Make Sure Everything Ends on the Same Column

  mov     A,#' '                ;  Put in three Blanks
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar

  djnz    R3,HUA_Loop2

HUA_Skip2:                      ;  Now, Display the Instruction

  lcall   InsDisplay            ;  Display the Instruction

HUA_Done:                       ;  Finished Putting in the Parameter Values

  mov     A,#CRLF
  lcall   SendMsg

  dec     R5                    ;  Have to Make a "djnz R5,HUA_Loop Instruction"
  mov     A,R5
  jz      HUA_End
  ajmp    HUA_Loop
HUA_End:
  ljmp    Loop

Not_Unassemble:                 ;  Modify the Control Store at Specified Address

  mov     A,@R0
  xrl     A,#'C'
  jz      Change
  ajmp    Not_Change
Change:

  cjne    R1,#0FFh,C_RegWrite   ;  Is there a Valid Address to Write to?

  mov     A,#4                  ;  No Address Specified
  lcall   SendMsg

  ljmp    Loop

C_RegWrite:

  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

CAddr_Loop:                     ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,CHex_Skip1      ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
CHex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     CAddr_Loop            ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      CAddr_OK

  mov     A,#3                  ;  Put in the Invalid Address Error
  lcall   SendMsg

  ljmp    Loop

CAddr_OK:                       ;  The Address is OK - Now Get the New Value

  mov     A,DPH                 ;  Put the Address in the External RAM
  add     A,#040h               ;   Starts at 16K
  mov     DPH,A

  mov     A,#2                  ;  Put in the Prompt Message
  lcall   SendMsg

  mov     A,#0                  ;  Get the Hex Value
  lcall   GetHex
  jnc     CReg_Check            ;  Check the Register to Read
  ljmp    Change_End            ;  If Nothing Entered, then Skip Over Write
CReg_Check:  

  movx @DPTR,A                  ;  Save the New Value in External Memory

  mov     A,#CRLF
  lcall   SendMsg

  ljmp    Loop

Not_Change:                     ;  Display Scratchpad RAM/Special Function Registers

  mov     A,@R0
  xrl     A,#'R'
  jz      RegRead
  ljmp    Not_RegRead
RegRead:

  mov     A,#6                  ;  Display the Scratchpad Registers First
  lcall   SendMsg

  mov     A,#0                  ;  Display the Saved Scratchpad RAM Registers First
  lcall   SendHex

  mov     A,#7                  ;  ":  "
  lcall   SendMsg

  mov     R0,#SavedRegs         ;  use "R0" as the Index Into Saved Regs
  mov     R1,#8                 ;  Do 8 Registers First
  
RR_Loop1:                       ;  Do the Line

  mov     A,@R0                 ;  Get the First Byte
  lcall   SendHex

  mov     A,#' '
  lcall   SendChar

  inc     R0                    ;  Point to the Next Register

  djnz    R1,RR_Loop1

  mov     R0,#008h              ;  Now, Display the Rest of the Scratchpad RAM

  mov     R1,#008h              ;  End with the Rest of the First Line
 
RR_Loop2:

  mov     A,@R0
  lcall   SendHex

  mov     A,#' '
  lcall   SendChar

  inc     R0

  djnz    R1,RR_Loop2

  mov     2,R0                  ;  Save Address for Later

  mov     A,#CRLF               ;  Now, Go to another line
  lcall   SendMsg

  mov     A,R2                  ;  Get the Address

  jb      ACC.7,ScratchDone     ;  If Bit 7 is Set, then Scratchpad is Done

  mov     A,R2                  ;  Otherwise, Start a New Line
  lcall   SendHex

  mov     A,#7
  lcall   SendMsg

  mov     0,R2                  ;  Do the Next Line
  mov     R1,#16                ;  Each Line 16 Bytes Long

  ajmp    RR_Loop2

ScratchDone:                    ;  Scratchpad Registers Displayed

  mov     A,#CRLF               ;  Space Out by One Line
  lcall   SendMsg

  mov     A,#22                 ;  First, Display the "B" Register
  lcall   SendMsg

  mov     R0,#08Eh
  mov     A,@R0
  lcall   SendHex               ;  Send the "B" Register Contents

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#21                 ;  Next, Display the PSW Contents
  lcall   SendMsg

  mov     R0,#08Ch
  mov     A,@R0
  lcall   SendBit

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#8                  ;  Display the SP Contents
  lcall   SendMsg

  mov     R0,#088h
  mov     A,@R0
  lcall   SendHex

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#9                  ;  Display the DPTR Contents
  lcall   SendMsg

  mov     R0,#08Ah
  mov     A,@R0
  lcall   SendHex
  dec     R0
  mov     A,@R0
  lcall   SendHex

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#19                  ;  Display the Interrupt Controller Contents
  lcall   SendMsg

  mov     R0,#08Bh
  mov     A,@R0
  lcall   SendBit

  mov     A,#' '
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar

  mov     A,#20
  lcall   SendMsg

  mov     A,IP
  lcall   SendBit

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#10                  ;  Display the PCON Contents
  lcall   SendMsg

  mov     A,PCON
  lcall   SendBit

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#15                  ;  Display I/O Port Values
  lcall   SendMsg

  mov     A,P1
  lcall   SendBit

  mov     A,#' '
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar

  mov     A,#16
  lcall   SendMsg

  mov     A,P3
  lcall   SendBit

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#11                  ;  Display Timer Control Values
  lcall   SendMsg

  mov     A,TCON
  lcall   SendBit

  mov     A,#' '
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar

  mov     A,#12
  lcall   SendMsg

  mov     A,TMOD
  lcall   SendBit

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#13                  ;  Display the Timer 0 Contents
  lcall   SendMsg

  mov     A,TH0
  lcall   SendHex
  mov     A,TL0
  lcall   SendHex

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#14                  ;  Display the Timer 1 Contents
  lcall   SendMsg

  mov     A,TH1
  lcall   SendHex
  mov     A,TL1
  lcall   SendHex

  mov     A,#CRLF
  lcall   SendMsg

  mov     A,#17                  ;  Display Serial Port Values
  lcall   SendMsg

  mov     A,SCON
  lcall   SendBit

  mov     A,#' '
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar

  mov     A,#18
  lcall   SendMsg

  mov     A,SBUF
  lcall   SendHex

  mov     A,#CRLF
  lcall   SendMsg

  ljmp    Loop

Not_RegRead:                    ;  Write to the Scratchpad RAM/Special Function Register

  mov     A,@R0
  xrl     A,#'W'
  jz      NowDo_RegWrite
  ljmp    Not_RegWrite
NowDo_RegWrite:

  cjne    R1,#0FFh,RegWrite     ;  Is there a Valid Address to Write to?

  mov     A,#4                  ;  No Address Specified
  lcall   SendMsg

  ljmp    Loop

RegWrite:

  mov     A,R1                  ;  Figure Out if Valid Address
  anl     A,#00Fh               ;  Clear the Buffer Address
  clr     C
  subb    A,R2                  ;  Check the End Address
  add     A,#2
  jc      Do_RegWrite           ;  If Carry Set, then Difference is 1 or 2

  mov     A,#3                  ;  Invalid Address
  lcall   SendMsg

  ljmp    Loop

Do_RegWrite:

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,Hex_Skip1	;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
Hex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     R0,A

  mov     A,R1                  ;  Do we Have one or two Characters?
  anl     A,#00Fh
  clr     C
  subb    A,R2
  xrl     A,#0FFh               ;  Invert the Result
  inc     A
  xrl     A,#2
  jnz     HaveAddr

  mov     A,R0
  swap A                        ;  Byte in "A" is Actually High Nybble
  mov     R0,A

  inc     R1                    ;  Get the Second Nybble of the Address
  mov     A,@R1
  clr     C
  jnb     ACC.6,Hex_Skip2
  subb    A,#7
Hex_Skip2:
  subb    A,#030h

  orl     A,R0
  mov     R0,A

HaveAddr:                       ;  Have a Valid Address in R0, Now, Get the New Value

  mov     5,R0                  ;  Save the Address 

  mov     A,#2                  ;  Put in the Prompt Message
  lcall   SendMsg

  mov     0,R5                  ;  Restore the Address

  mov     A,#0                  ;  Get the Hex Value
  lcall   GetHex
  jnc     Reg_Check             ;  Check the Register to Read
  ljmp    Change_End            ;  If Nothing Entered, then Skip Over Write
Reg_Check:  

  mov     R1,A                  ;  Save the New Value

  mov     A,R0                  ;  Get the Address to Check
  clr     C
  subb    A,#080h               ;  Do We Write to a Scratchpad Register or SFR?
  jnc     SFR_Write             ;  No Carry, A special Function Register

  mov     A,R0                  ;  Now, Do we write to the Bank Registers?
  clr     C
  subb    A,#008h               ;  Do We have a Bank Register?
  jnc     Scratch_Write         ;  No, Can Write Directly

  mov     A,R0
  add     A,#SavedRegs          ;  Put the Bank Registers in the Saved Register Area
  mov     R0,A

Scratch_Write:                  ;  Write to Scratchpad RAM

  mov     A,R1
  mov     @R0,A

  mov     A,#5                  ;  New Line and End
  lcall   SendMsg

  ljmp    Loop

SFR_Write:                      ;  Write to the Specified SFR

  mov     A,R0                  ;  Look At SP
  xrl     A,#081h
  jnz     SFRW_NotSP

  mov     R0,#088h              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotSP:

  mov     A,R0                  ;  Look At DPL
  xrl     A,#082h
  jnz     SFRW_NotDPL

  mov     R0,#089h              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotDPL:

  mov     A,R0                  ;  Look At DPH
  xrl     A,#083h
  jnz     SFRW_NotDPH

  mov     R0,#08Ah              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotDPH:

  mov     A,R0                  ;  Look At PCON
  xrl     A,#087h
  jnz     SFRW_NotPCON

  mov     087h,R1               ;  Write Directly to "PCON"

  ljmp    Change_End

SFRW_NotPCON:

  mov     A,R0                  ;  Look At TCON
  xrl     A,#088h
  jnz     SFRW_NotTCON

  mov     088h,R1               ;  Write Directly to "TCON"

  ljmp    Change_End

SFRW_NotTCON:

  mov     A,R0                  ;  Look At TMOD
  xrl     A,#089h
  jnz     SFRW_NotTMOD

  mov     089h,R1               ;  Write Directly to "TMOD"

  ljmp    Change_End

SFRW_NotTMOD:

  mov     A,R0                  ;  Look At TL0
  xrl     A,#08Ah
  jnz     SFRW_NotTL0

  mov     08Ah,R1               ;  Write Directly to "TL0"

  ljmp    Change_End

SFRW_NotTL0:

  mov     A,R0                  ;  Look At TL1
  xrl     A,#08Bh
  jnz     SFRW_NotTL1

  mov     08Bh,R1               ;  Write Directly to "TL1"

  ljmp    Change_End

SFRW_NotTL1:

  mov     A,R0                  ;  Look At TH0
  xrl     A,#08Ch
  jnz     SFRW_NotTH0

  mov     08Ch,R1               ;  Write Directly to "TH0"

  ljmp    Change_End

SFRW_NotTH0:

  mov     A,R0                  ;  Look At TH1
  xrl     A,#08Dh
  jnz     SFRW_NotTH1

  mov     08Dh,R1               ;  Write Directly to "TH1"

  ljmp    Change_End

SFRW_NotTH1:

  mov     A,R0                  ;  Look At P1
  xrl     A,#090h
  jnz     SFRW_NotP1

  mov     090h,R1               ;  Write Directly to "P1"

  ljmp    Change_End

SFRW_NotP1:

  mov     A,R0                  ;  Look At SCON
  xrl     A,#098h
  jnz     SFRW_NotSCON

  mov     098h,R1               ;  Write Directly to "SCON"

  ljmp    Change_End

SFRW_NotSCON:

  mov     A,R0                  ;  Look At SBUF
  xrl     A,#099h
  jnz     SFRW_NotSBUF

  mov     099h,R1               ;  Write Directly to "SBUF"

  ljmp    Change_End

SFRW_NotSBUF:

  mov     A,R0                  ;  Look At IE
  xrl     A,#0A8h
  jnz     SFRW_NotIE

  mov     R0,#08Bh              ;  Save IE in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotIE:

  mov     A,R0                  ;  Look At P3
  xrl     A,#0B0h
  jnz     SFRW_NotP3

  mov     0B0h,R1               ;  Write Directly to "P3"

  ljmp    Change_End

SFRW_NotP3:

  mov     A,R0                  ;  Look At IP
  xrl     A,#0B8h
  jnz     SFRW_NotIP

  mov     0B8h,R1               ;  Write Directly to "IP"

  ljmp    Change_End

SFRW_NotIP:

  mov     A,R0                  ;  Look At PSW
  xrl     A,#0D0h
  jnz     SFRW_NotPSW

  mov     R0,#08Ch              ;  Save PSW in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotPSW:

  mov     A,R0                  ;  Look At ACC
  xrl     A,#0E0h
  jnz     SFRW_NotACC

  mov     R0,#08Dh              ;  Save ACC in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotACC:

  mov     A,R0                  ;  Look At "B"
  xrl     A,#0F0h
  jnz     SFRW_NotB

  mov     R0,#08Eh              ;  Save "B" in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ljmp    Change_End

SFRW_NotB:

  mov     A,#3                  ;  Not a Valid AT89Cx051 Message
  lcall   SendMsg

  ljmp    Loop

Change_End:

  mov     A,#CRLF               ;  Start a New Line
  lcall   SendMsg

  ljmp    Loop

Not_RegWrite:                   ;  Toggle Breakpoint at the Specified Address

  mov     A,@R0
  xrl     A,#'B'
  jz      Breakpoint
  ljmp    Not_BreakPoint
Breakpoint:

  cjne    R1,#0FFh,BPWrite      ;  Is there a Valid Address to Write to?

  mov     A,#4                  ;  No Address Specified
  lcall   SendMsg

  ljmp    Loop

BPWrite:

  mov     A,R1                  ;  Figure Out if Valid Address
  anl     A,#00Fh               ;  Clear the Buffer Address
  clr     C
  subb    A,R2                  ;  Check the End Address
  add     A,#2
  jc      Do_BPWrite            ;  If Carry Set, then Difference is 1 or 2

  mov     A,#3                  ;  Invalid Address
  lcall   SendMsg

  ljmp    Loop

Do_BPWrite:
  
  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

BP_Loop:                        ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 Bits for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,BP_Skip1        ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
BP_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     BP_Loop               ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      BPAddr_OK

  mov     A,#3                  ;  Put in the Invalid Address Error
  lcall   SendMsg

  ljmp    Loop

BPAddr_OK:

BP0_Check:                      ;  Start Here with Break0

  mov     R0,#Break0
  cjne    @R0,#0,BP1_Check      ;  Looking for an open Space

BP0_Check1:

  mov     R0,#Break1            ;  Compare to the Other Breakpoints
  cjne    @R0,#0FFh,BP0_Check2

  inc     R0                    ;  Get the Breakpoint1 Address
  mov     2,@R0
  inc     R0
  mov     3,@R0

  mov     A,R2                  ;  Find the Difference
  clr     C
  subb    A,DPL
  mov     R2,A
  mov     A,R3
  subb    A,DPH
  mov     R3,A

  jnb     ACC.7,BP0_Check1Do

  mov     A,#0                  ;  Negative Value, Make Positive
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,#0
  subb    A,R3
  mov     R3,A

BP0_Check1Do:                   ;  Is the Proposed > 8 Away?  

  cjne    R3,#0,BP0_Check2      ;  If Not Zero, then No Problems
  mov     A,R2
  clr     C
  subb    A,#9                  ;  Must be at least 9 Away
  jnc     BP0_Check2

  mov     A,#7                  ;  Put in the Error Message
  lcall   SendMsg2

  ljmp    Loop

BP0_Check2:

  mov     R0,#Break2            ;  Compare to the Other Breakpoints
  cjne    @R0,#0FFh,BP0_Save    ;   If Nothing Loaded, Then Save as BP0

  inc     R0                    ;  Get the Breakpoint1 Address
  mov     2,@R0
  inc     R0
  mov     3,@R0

  mov     A,R2                  ;  Find the Difference
  clr     C
  subb    A,DPL
  mov     R2,A
  mov     A,R3
  subb    A,DPH
  mov     R3,A

  jnb     ACC.7,BP0_Check2Do

  mov     A,#0                  ;  Negative Value, Make Positive
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,#0
  subb    A,R3
  mov     R3,A

BP0_Check2Do:                   ;  Is the Proposed > 8 Away?  

  cjne    R3,#0,BP0_Save        ;  If Not Zero, then No Problems
  mov     A,R2
  clr     C
  subb    A,#9                  ;  Must be at least 9 Away
  jnc     BP0_Save

  mov     A,#7                  ;  Put in the Error Message
  lcall   SendMsg2

  ljmp    Loop

BP0_Save:                       ;  Can Save the BP Address

  mov     R0,#Break0
  mov     @R0,#0FFh             ;  Mark that the Breakpoint is used

  inc     R0
  mov     @R0,DPL               ;  Save the Breakpoint addresses
  inc     R0
  mov     @R0,DPH

  ljmp    Loop

BP1_Check:                      ;  Can we Put the BP in Break1?

  mov     R0,#Break1
  cjne    @R0,#0,BP2_Check

  mov     R0,#Break0            ;  Compare to the Other Breakpoints
  inc     R0                    ;  Get the Breakpoint1 Address
  mov     2,@R0
  inc     R0
  mov     3,@R0

  mov     A,R2                  ;  Find the Difference
  clr     C
  subb    A,DPL
  mov     R2,A
  mov     A,R3
  subb    A,DPH
  mov     R3,A

  jnb     ACC.7,BP1_Check0Do

  mov     A,#0                  ;  Negative Value, Make Positive
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,#0
  subb    A,R3
  mov     R3,A

BP1_Check0Do:                   ;  Is the Proposed > 8 Away?  

  cjne    R3,#0,BP1_Check2      ;  If Not Zero, then No Problems
  mov     A,R2
  clr     C
  subb    A,#9                  ;  Must be at least 9 Away
  jnc     BP1_Check2

  mov     A,#7                  ;  Put in the Error Message
  lcall   SendMsg2

  ljmp    Loop

BP1_Check2:

  mov     R0,#Break2            ;  Compare to the Other Breakpoints
  cjne    @R0,#0FFh,BP1_Save    ;   If Nothing Loaded, Then Save as BP0

  inc     R0                    ;  Get the Breakpoint1 Address
  mov     2,@R0
  inc     R0
  mov     3,@R0

  mov     A,R2                  ;  Find the Difference
  clr     C
  subb    A,DPL
  mov     R2,A
  mov     A,R3
  subb    A,DPH
  mov     R3,A

  jnb     ACC.7,BP1_Check2Do

  mov     A,#0                  ;  Negative Value, Make Positive
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,#0
  subb    A,R3
  mov     R3,A

BP1_Check2Do:                   ;  Is the Proposed > 8 Away?  

  cjne    R3,#0,BP1_Save        ;  If Not Zero, then No Problems
  mov     A,R2
  clr     C
  subb    A,#9                  ;  Must be at least 9 Away
  jnc     BP1_Save

  mov     A,#7                  ;  Put in the Error Message
  lcall   SendMsg2

  ljmp    Loop

BP1_Save:                       ;  Can Save the BP Address

  mov     R0,#Break1
  mov     @R0,#0FFh             ;  Mark that the Breakpoint is used

  inc     R0
  mov     @R0,DPL               ;  Save the Breakpoint addresses
  inc     R0
  mov     @R0,DPH

  ljmp    Loop

BP2_Check:

  mov     R0,#Break2
  cjne    @R0,#0,No_BP          ;  If we're Here, Any Breakpoints Available?  

  mov     R0,#Break0            ;  Compare to the Other Breakpoints
  inc     R0                    ;  Get the Breakpoint1 Address
  mov     2,@R0
  inc     R0
  mov     3,@R0

  mov     A,R2                  ;  Find the Difference
  clr     C
  subb    A,DPL
  mov     R2,A
  mov     A,R3
  subb    A,DPH
  mov     R3,A

  jnb     ACC.7,BP2_Check0Do

  mov     A,#0                  ;  Negative Value, Make Positive
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,#0
  subb    A,R3
  mov     R3,A

BP2_Check0Do:                   ;  Is the Proposed > 8 Away?  

  cjne    R3,#0,BP2_Check1      ;  If Not Zero, then No Problems
  mov     A,R2
  clr     C
  subb    A,#9                  ;  Must be at least 9 Away
  jnc     BP2_Check1

  mov     A,#7                  ;  Put in the Error Message
  lcall   SendMsg2

  ljmp    Loop

BP2_Check1:

  mov     R0,#Break1            ;  Compare to the Other Breakpoints
  inc     R0                    ;  Get the Breakpoint1 Address
  mov     2,@R0
  inc     R0
  mov     3,@R0

  mov     A,R2                  ;  Find the Difference
  clr     C
  subb    A,DPL
  mov     R2,A
  mov     A,R3
  subb    A,DPH
  mov     R3,A

  jnb     ACC.7,BP2_Check1Do

  mov     A,#0                  ;  Negative Value, Make Positive
  clr     C
  subb    A,R2
  mov     R2,A
  mov     A,#0
  subb    A,R3
  mov     R3,A

BP2_Check1Do:                   ;  Is the Proposed > 8 Away?  

  cjne    R3,#0,BP2_Save        ;  If Not Zero, then No Problems
  mov     A,R2
  clr     C
  subb    A,#9                  ;  Must be at least 9 Away
  jnc     BP2_Save

  mov     A,#7                  ;  Put in the Error Message
  lcall   SendMsg2

  ljmp    Loop

BP2_Save:                       ;  Can Save the BP Address

  mov     R0,#Break2
  mov     @R0,#0FFh             ;  Mark that the Breakpoint is used

  inc     R0
  mov     @R0,DPL               ;  Save the Breakpoint addresses
  inc     R0
  mov     @R0,DPH

  ljmp    Loop

No_BP:                          ;  No Way to Put in the Breakpoint

  mov     A,#8
  lcall   SendMsg2

  ljmp    Loop

Not_BreakPoint:                 ;  List the Breakpoints set in the Emulator?

  mov     A,@R0
  xrl     A,#'L'
  jz      Do_Break
  ljmp    Check_Help
Do_Break:

  mov     TL2,#0                ;  Use TL2 to Count the available Breakpoints

Break_Check0:

  mov     R0,#Break0            ;  Check the first Breakpoint
  cjne    @R0,#0FFh,Break_Check1;  If Not Equal To Zero, Check the Next Breakpoint

  inc     TL2

  mov     A,#3
  lcall   SendMsg2
  mov     A,#'1'
  lcall   SendChar
  mov     A,#4
  lcall   SendMsg2

  mov     R0,#(Break0 + 2)
  mov     A,@R0
  lcall   SendHex
  mov     R0,#(Break0 + 1)
  mov     A,@R0
  lcall   SendHex

  mov     A,#'h'
  lcall   SendChar
  mov     A,#CRLF
  lcall   SendMsg

Break_Check1:

  mov     R0,#Break1            ;  Check the Second Breakpoint
  cjne    @R0,#0FFh,Break_Check2;  If Not Equal To Zero, Check the Next Breakpoint

  inc     TL2

  mov     A,#3
  lcall   SendMsg2
  mov     A,#'2'
  lcall   SendChar
  mov     A,#4
  lcall   SendMsg2

  mov     R0,#(Break1 + 2)
  mov     A,@R0
  lcall   SendHex
  mov     R0,#(Break1 + 1)
  mov     A,@R0
  lcall   SendHex

  mov     A,#'h'
  lcall   SendChar
  mov     A,#CRLF
  lcall   SendMsg

Break_Check2:

  mov     R0,#Break2            ;  Check the first Breakpoint
  cjne    @R0,#0FFh,Break_Done  ;  If Not Equal To Zero, Then See if Any Set

  inc     TL2

  mov     A,#3
  lcall   SendMsg2
  mov     A,#'3'
  lcall   SendChar
  mov     A,#4
  lcall   SendMsg2

  mov     R0,#(Break2 + 2)
  mov     A,@R0
  lcall   SendHex
  mov     R0,#(Break2 + 1)
  mov     A,@R0
  lcall   SendHex

  mov     A,#'h'
  lcall   SendChar
  mov     A,#CRLF
  lcall   SendMsg

Break_Done:                     ;  Breakpoint Checking Done, If Zero Set, 
                                ;   Then put in Message

  mov     A,TL2
  cjne    A,#0,Break_MsgDone

  mov     A,#5
  lcall   SendMsg2

Break_MsgDone:

  ljmp    Loop

Check_Help:                     ;  Display the Help Message

  mov     A,@R0                 ;  Check for "H"elp Selected
  xrl     A,#'H'
  jz      Help

  mov     A,#1                  ;  Return "Invalid Message"
  lcall   SendMsg

Help:

  lcall   DisplayHelp1          ;  As the First Thing, Display Help
  lcall   DisplayHelp2
  lcall   DisplayHelp3

  ljmp    Loop  


;  Subroutines
Prompt:                         ;  Display a Prompt for the Program

  mov     R0,#08Dh              ;  Point to the saved Accumulator
  mov     A,@R0
  lcall   SendHex

  mov     A,#' '
  lcall   SendChar

  mov     R0,#08Ch              ;  Point to the Saved PSW
  mov     A,@R0
  mov     C,ACC.7               ;  Save the Carry Flag
  cpl     C                     ;  Want Upper Case if Set
  mov     A,#'C'
  mov     ACC.5,C
  lcall   SendChar

  mov     A,#' '
  lcall   SendChar

  mov     R0,#08Bh              ;  Point to the Saved IE
  mov     A,@R0
  mov     C,ACC.7               ;  Save the EA Flag
  cpl     C
  mov     A,#'I'
  mov     ACC.5,C
  lcall   SendChar

  mov     A,#' '
  lcall   SendChar

  mov     R0,#(ProgC + 1)       ;  Now, Get the Value at the Current Program Counter
  mov     A,@R0
  mov     DPH1,@R0
  lcall   SendHex
  mov     DPH,@R0
  mov     A,DPH
  add     A,#040h
  mov     DPH,A
  dec     R0
  mov     A,@R0
  mov     DPL1,@R0
  lcall   SendHex
  mov     DPL,@R0

  mov     A,#':'
  lcall   SendChar

  mov     R0,#(Break0)          ;  Look for Breakpoints and Mark
  cjne    @R0,#0FFh,Prompt_B1

  inc     R0
  mov     2,@R0                 ;  Load the Low Byte of the Breakpoint
  inc     R0
  mov     3,@R0                 ;  Load the High Byte of the Breakpoint

  mov     A,DPL1                ;  Now, look for a Match
  xrl     A,R2
  mov     R2,A
  mov     A,DPH1
  xrl     A,R3
  orl     A,R2

  jnz     Prompt_B1             ;  If Not Equal to Zero, then No Match
  
Prompt_Match:

  mov     A,#'*'                ;  Mark the Match

  sjmp    Prompt_BPDisplay

Prompt_B1:                      ;  Now, See if Break1 is a Match

  mov     R0,#(Break1)          ;  Look for Breakpoints and Mark
  cjne    @R0,#0FFh,Prompt_B2

  inc     R0
  mov     2,@R0                 ;  Load the Low Byte of the Breakpoint
  inc     R0
  mov     3,@R0                 ;  Load the High Byte of the Breakpoint

  mov     A,DPL1                ;  Now, look for a Match
  xrl     A,R2
  mov     R2,A
  mov     A,DPH1
  xrl     A,R3
  orl     A,R2

  jz      Prompt_Match          ;  If Not Equal to Zero, then No Match

Prompt_B2:                      ;  Now, See if Break2 is a Match

  mov     R0,#(Break1)          ;  Look for Breakpoints and Mark
  cjne    @R0,#0FFh,Prompt_NoMatch

  inc     R0
  mov     2,@R0                 ;  Load the Low Byte of the Breakpoint
  inc     R0
  mov     3,@R0                 ;  Load the High Byte of the Breakpoint

  mov     A,DPL1                ;  Now, look for a Match
  xrl     A,R2
  mov     R2,A
  mov     A,DPH1
  xrl     A,R3
  orl     A,R2

  jz      Prompt_Match          ;  If Not Equal to Zero, then No Match

Prompt_NoMatch:                 ;  No Match, Just Put in a Blank

  mov     A,#' '
Prompt_BPDisplay:
  lcall   SendChar

  movx A,@DPTR                  ;  Get the Byte at the Current Instruction
  mov     R4,A

  lcall   InsLength             ;  Get the Number of Instructions

  dec     A                     ;  Get all the Bytes
  jz      Prompt_Ins

  mov     R3,A                  ;  Save the Count

Prompt_Loop:                    ;  Loop Here to Read Each Byte

  inc     DPTR                  ;  Increment the External Memory Pointer

  mov     A,DPL1                ;  Shift Up the Data
  mov     DPH1,A

  movx A,@DPTR

  mov     DPL1,A                ;  Save the Byte

  djnz    R3,Prompt_Loop

Prompt_Ins:                     ;  Now, Have the Instruction in R4/DPL1/DPH1

  mov     TL2,#0                ;  Use TL2 as a Counter

  lcall   InsDisplay            ;  Display it
  
Prompt_Loop2:                   ;  Put in Blanks Until TL2 == 15

  mov     A,TL2                 ;  Have we Output 15 Characters?  
  xrl     A,#16
  jz      Prompt_End

  mov     A,#' '
  lcall   SendChar

  sjmp    Prompt_Loop2

Prompt_End:

  mov     A,#'>'
  lcall   SendChar
  mov     A,#' '
  lcall   SendChar

  ret


GetHex:                         ;  Get a Hex Byte from the User

  mov     R2,#0                 ;  Character to Return
  mov     R3,A                  ;  Save the Return Type 
                                ;   !0 - Space or Enter
                                ;    0 - Just Enter
  mov     R4,#0                 ;  Number of Characters Read

GetHex_Loop:                    ;  Loop Here for Each Character

  lcall   GetChar               ;  Get Character/Wait 10 msecs

  jc      GetHex_Loop           ;  If Carry Set, Nothing to Check

  mov     R5,A                  ;  Save the Character Returned

  xrl     A,#00Dh               ;  Was Enter Pressed?  
  jnz     GetHex_NotEnter

  lcall   GetChar               ;  Wait for LF (if Coming

GetHex_Enter:

  mov     A,R2                  ;  Get the Number to Return

  clr     C                     ;  Assume Something Returned, Reset Carry
  cjne    R4,#0,GetHex_End      ;  Do we Have a number to Return?
  setb    C                     ;  Nothing Entered, Set Carry 
GetHex_End:

  ret

GetHex_NotEnter:                ;  Not Enter, What Else is it?

  mov     A,R5                  ;  Do We have a Blank?  
  xrl     A,#' '
  jnz     GetHex_NotBlank

  cjne    R3,#0,GetHex_Enter    ;  Can we Return on a Blank?
  
  sjmp    GetHex_Loop           ;  Else, Ignore a Loop

GetHex_NotBlank:                ;  Now, Look For Valid Hex

  mov     A,R5                  ;  Do we Have a Backspace?
  xrl     A,#BS
  jnz     GetHex_NotBS

  mov     A,R4                  ;  Do we Have a Character to Take Away?
  jz      GetHex_Loop

  mov     A,#BS                 ;  Return the BS to the Host
  lcall   SendChar

  mov     A,R2                  ;  Take One Digit out of the Return Value
  swap A
  anl     A,#00Fh               ;  Clear the Upper 4 Bits of Shifted Result
  mov     R2,A

  dec     R4

  sjmp    GetHex_Loop

GetHex_NotBS:                   ;  Have Ascii Character?

  mov     A,R4                  ;  Do we Have 2 Characters Already?  
  xrl     A,#2
  jz      GetHex_Loop           ;  Yes, Ignore the Character

  mov     A,R5                  ;  Do Hex Comparison
  clr     C
  subb    A,#'0'
  jc      GetHex_Loop           ;  Less than '0', Not Valid Hex

  mov     A,R5
  clr     C
  subb    A,#('9' + 1)
  jc      GetHex_HaveHex

  mov     A,R5                  ;  "A" to "F"?
  clr     C
  subb    A,#'A'
  jc      GetHex_Loop

  mov     A,R5
  clr     C
  subb    A,#('F' + 1)
  jc      GetHex_HaveHex

  mov     A,R5                  ;  "a" to "f"?
  clr     C
  subb    A,#'a'
  jc      GetHex_Loop

  mov     A,R5
  clr     C
  subb    A,#('f' + 1)
  jnc     GetHex_Loop

  mov     A,R5
  anl     A,#05Fh               ;  Convert Number to Upper Case
  mov     R5,A

GetHex_HaveHex:                 ;  Have a Hex Character in R5, Add to R2

  mov     A,R5                  ;  Send the Character Back to the Host
  lcall   SendChar

  mov     A,R2                  ;  Prepare R2 for the Character
  swap A
  mov     R2,A

  mov     A,R5
  clr	  C			
  jnb	  ACC.6,GetHex_Skip  		;  If Bit 6 of the ASCII Value is Set
  subb    A,#7				;   Subtract Difference between 'A' & '9'
GetHex_Skip:
  subb    A,#030h			;  Take away '0' to get the Nybble

  orl     A,R2                          ;  Add the Bits to R2
  mov     R2,A

  inc     R4                    ;  Increment the Number of Characters  

  sjmp    GetHex_Loop


GetCommand:                     ;  Get the Command from the User

  mov     R2,#0                 ;  R3 Counts the Number of Characters Entered
  mov     R3,#' '               ;  R4 Has the Last Character Entered
                                ;   Use "Blank" incase a Character Entered
  mov     R4,#0FFh              ;  R5 Has the Position of the First Parameter
  mov     R5,#0FFh              ;  R6 Has the Position of the Second Parameter

GetCommand_Loop:                ;  Loop Around Here

  lcall   GetChar               ;  Get Character/Wait 10 msecs

  jc      GetCommand_Loop       ;  If Carry Set, Nothing to Check

  cjne    A,#CR,GetCommand_Test ;  Look for a Carriage Return

  lcall   GetChar               ;  Wait for/Skip the LF

  mov     A,#CRLF               ;  Start a New Line
  lcall   SendMsg

  mov     0,R4                  ;  Save the Parameter Position Values
  mov     1,R5

  mov     A,R4                  ;  Do We Have Something to Change?
  xrl     A,#0FFh
  jz      GetCommand_End

  mov     A,R4                  ;  Pass Back the Parameters
  add     A,#Buffer
  mov     R0,A

  mov     A,R5                  ;  Do We Have Something to Change?
  xrl     A,#0FFh
  jz      GetCommand_End

  mov     A,R5
  add     A,#Buffer
  mov     R1,A

GetCommand_End:

  ret

GetCommand_Test:                ;  Now, Process the Character

  mov     R1,A                  ;  Save the Character

  clr     C                     ;  Do we Have a Special Character?
  subb    A,#' '                ;  Is there anything less than ' '?

  jnc     GetCommand_Char       ;  No, Process the Character

  cjne    R1,#BS,GetCommand_Loop;  If it's not BackSpace, Display the Character
  
  cjne    R2,#0,GetCommand_BS   ;  If No Characters, Then Skip
  sjmp    GetCommand_Loop
GetCommand_BS:

  mov     A,R1                  ;  Send the BackSpace Character
  lcall   SendChar

  dec     R2                    ;  Decrement the Displayed Characters

  mov     A,R2                  ;  Now, Do We Have to Clear the Parameter Pos'ns?
  xrl     A,R5                  ;  Check the Second Parameter
  jnz     GetCommand_BSFirst

  mov     R5,#0FFh              ;  Reset the Second Parameter

GetCommand_BSFirst:             ;  Check the position against the First Parameter

  mov     A,R2
  xrl     A,R4
  jnz     GetCommand_BSNew      

  mov     R4,#0FFh

GetCommand_BSNew:               ;  Now, Finish off with Resetting the Last Character

  mov     R3,#' '               ;  Get the Last Character
  
  mov     A,R2
  jz      GetCommand_Loop       ;  If At the Start of the Row, Return

  add     A,#Buffer             ;  Get the Actual Position from the Buffer
  dec     A                     ;   Of the Previous Character

  mov     R0,A                  ;  Get the Character and Save it in R3
  mov     A,@R0
  mov     R3,A

  sjmp    GetCommand_Loop

GetCommand_Char:                ;  Process the Character

  cjne    R2,#10,GetCommand_OK  ;  If 10 Characters In, 
  sjmp    GetCommand_Loop
GetCommand_OK:

  mov     A,R1                  ;  Make Sure the Character is Upper Case
  clr     C
  subb    A,#'a'                ;  Are we less than 'a'?
  jc      GetCommand_UC

  mov     A,R1                  ;  Are we Greater than 'z'?
  clr     C
  subb    A,#('z' + 1)
  jnc     GetCommand_UC

  mov     A,R1
  anl     A,#05Fh               ;  Lower Case, Convert to Upper
  mov     R1,A

GetCommand_UC:                  ;  Have an Upper Case Character

  cjne    R3,#' ',GetCommand_Do ;  If Previous == ' ', Save the Parameter?
  mov     A,R1
  xrl     A,#' '
  jz      GetCommand_Do         ;  If Current is Also ' ', Don't Save Parameter

  cjne    R4,#0FFh,GetCommand_2nd

  mov     4,R2                  ;  Save the Position for the First Parameter

  sjmp    GetCommand_Do

GetCommand_2nd:                 ;  Save the Parameter as 2nd?

  cjne    R5,#0FFh,GetCommand_Do

  mov     5,R2                  ;   Yes

GetCommand_Do:                  ;  Save the Character and Wait for Next

  cjne    R5,#0FFh,GetCommand_2ndHex

GetCommand_ActualDo:

  mov     A,R2                  ;  Save the Character
  add     A,#Buffer
  mov     R0,A
  mov     A,R1
  mov     @R0,A

  mov     A,R1
  mov     R3,A                  ;  Save the Character as Previous
  lcall   SendChar              ;  Send the Character to the Display

  inc     R2                    ;  Point to the Next Position

  ljmp    GetCommand_Loop

GetCommand_2ndHex:              ;  Second Parameter, Must Be Hex (and Max of 4 Chars)

  mov     A,R2                  ;  Do we Have 4 Characters?  
  clr     C
  subb    A,R5
  xrl     A,#3
  jnz     GetCommand_ActualDo   ;  No, Add to the Buffer

  ljmp    GetCommand_Loop       ;  Yes, Don't Add Any More 


DisplayHelp1:                   ;  Display the "Help" Menu

  mov     R0,#(HelpTable1 - GetHelp1)

DisplayHelp_Loop1:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp1:

  jz      DisplayHelp_End1      ;  '\0' is the End Character

  lcall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  sjmp    DisplayHelp_Loop1     ;  Loop Around to Display the Next Character
  
DisplayHelp_End1:

  ret

HelpTable1:                     ;  Messages to be Displayed
  db      CR,LF                 ;  Start a New Line
  db      'H        - Help, Display this List',CR,LF
  db      'T        - Reset',CR,LF
  db      'D        - Download a New Application',CR,LF
  db      'E [Addr] - Execute at the Optional Address',CR,LF
  db      '1 [Addr] - Single Step from the Optional Address',CR,LF,0
DisplayHelp2:                   ;  Display the "Help" Menu

  mov     R0,#(HelpTable2 - GetHelp2)

DisplayHelp_Loop2:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp2:

  jz      DisplayHelp_End2      ;  '\0' is the End Character

  lcall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  sjmp    DisplayHelp_Loop2     ;  Loop Around to Display the Next Character
  
DisplayHelp_End2:

  ret

HelpTable2:                     ;  Messages to be Displayed
  db      '         - If just a Carriage Return Received then Single Step',CR,LF
  db      'U [Addr] - Unassemble at the PC or Optional Address',CR,LF
  db      'C Addr   - Modify the Control Store at the Address',CR,LF
  db      'P Addr   - Change the Program Counter',CR,LF,0
DisplayHelp3:

  mov     R0,#(HelpTable2 - GetHelp2)

DisplayHelp_Loop3:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp3:

  jz      DisplayHelp_End3      ;  '\0' is the End Character

  lcall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  sjmp    DisplayHelp_Loop3     ;  Loop Around to Display the Next Character
  
DisplayHelp_End3:

  ret

HelpTable3:                     ;  Messages to be Displayed
  db      'W Addr   - Write to the Scratchpad RAM/SFR Address',CR,LF
  db      'R        - Display the Scratchpad RAM Registers',CR,LF
  db      'B Addr   - Set Breakpoint at the Specified Address',CR,LF
  db      'L        - List the Breakpoints',CR,LF,0


GetChar:                        ;  Wait for a Character and Return it in Accumulator

  mov  R7,#28                   ;  Delay 10 msecs for Test
  mov  R6,#0
GCLoop1:			;  Wait 10 msecs for a Start Bit
  jnb  SerIn,GCSkip
  djnz R6,GCLoop1
  djnz R7,GCLoop1

  setb C                        ;  Nothing Received, Set Carry to Indicate

  ret

GCSkip:                         ;  Have Something to Receive

  mov  R6,#9	                ;  Want to Read 9 Bits

  mov  R7,#70		        ;  Delay Less than 1/2 Bit to Make Up for Calls, Etc.
GCLoop2:
  djnz R7,GCLoop2

  jb   SerIn,GCLoop1	        ;  If "Glitch", keep Waiting

GCLoop:			        ;  Now, Read 9 Bits

  mov  C,SerIn	                ;  Read the Input Bit
  rrc  A

  mov  R7,#170  	        ;  Delay Loop for 104 usecs per bit
GCLoop3:
  djnz R7,GCLoop3
  nop
  nop

  djnz R6,GCLoop

  clr  C                        ;  Make Sure Carry is Clear

  ret


SendBit:                        ;  Send the Character in "A" as Binary

  mov    R2,A                   ;  Save the Character

  mov    R3,#8                  ;  Do 8 Bits

SB_Loop:                        ;  Loop Here

  mov    A,R2
  rlc A                         ;  Shift the Character Up and Put the LSB Bit in Carry
  mov    R2,A

  mov    A,#'0'                 ;  Figure out what Character
  addc   A,#0                   ;  Add the Carry to ASCII Zero

  lcall  SendChar               ;  Send the Bit to the Console

  djnz   R3,SB_Loop

  ret


SendHex:                        ;  Send the Character in "A" as Hex

  mov    R2,A                   ;  Save the Character

  swap A                        ;  Display the High Nybble First
  anl    A,#00Fh
  add    A,#036h	 	;  If ( A & 0x00Fh ) + 6 > 0x010h
  jnb    AC,SH_Skip1	 
  add    A,#7		 	;   Add 7 to Jump to "A" to "F"
SH_Skip1:
  subb   A,#6			;  Take Away Flag Set

  lcall  SendChar 

  mov    A,R2                   ;  Do the Lower Nybble   
  anl    A,#00Fh
  add    A,#036h	 	;  If ( A & 0x00Fh ) + 6 > 0x010h
  jnb    AC,SH_Skip2	 
  add    A,#7		 	;   Add 7 to Jump to "A" to "F"
SH_Skip2:
  subb   A,#6			;  Take Away Flag Set

  lcall  SendChar

  ret


SendDec:                        ;  Send the Value in "A" as Signed Decimal

  jnb    ACC.7,SD_Skip1         ;  Do We Have a negative?

  xrl    A,#0FFh
  inc    A                      ;  Convert to Positive
  mov    B,A                    ;  Save "ACC"

  mov    A,#'-'
  lcall  SendChar

  mov    A,B

SD_Skip1:                       ;  Do we Have 100s?  

  mov    B,#100
  div AB
  jz     SD_No100s              ;  If "A" is equal to Zero, then No

  add    A,#'0'                 ;  Convert to ASCII
  lcall  SendChar

  mov    A,B
  mov    B,#10
  div AB

  sjmp   SD_10s                 ;  Display the 10s

SD_No100s:                      ;  No 100s, do we Have 10s?

  mov    A,B
  mov    B,#10
  div AB

  jz     SD_No10s               ;  Do We have any 10s?  

SD_10s:                         ;  Display the 10s  

  add    A,#'0'
  lcall  SendChar

SD_No10s:                       ;  Display the 1s

  mov    A,B
  add    A,#'0'
  lcall  SendChar

  ret


SendChar:                       ;  Send the Serial Character
  mov	 R6,#10		        ;  We're Actually Sending 10 Bits

  clr	 C	       	        ;  First Bit is a "0"

SCLoop:			        ;  Loop Here for Each Bit

  mov	 SerOut,C               ;  Output the Bit in the Carry Flag

  mov	 R7,#170	        ;  Delay to 104 usecs (520 Cycles) per loop
SCLoop1:
  djnz	 R7,SCLoop1
  nop

  setb	 C		        ;  Shift the Next Bit to Carry
  rrc 	 A		        ;   And Load in a "1" for Stop Bit

  djnz	 R6,SCLoop

  inc    TL2                    ;  Increment a Timer for Character Counting

  ret


SendMsg:                        ;  Send the Specified Message to the Display

  mov     R1,A                  ;  Save the Message Number

  mov     R0,#0                 ;  Use R0 as the Offset Counter

SendMsg_Find:                   ;  Look for the Appropriate Message

  mov     A,R1                  ;  Are we Now At Zero?
  jz      SendMsg_Output        ;   If Count is Zero

  mov     A,#LOW(SendMsgTable - SendMsgFindGet)
  add     A,R0                  ;  Get Current Address

  movc A,@A+PC                  ;  Get the Character in the Displays
SendMsgFindGet:

  inc     R0                    ;  Point to the Next Character

  jnz     SendMsg_Find          ;  If NOT Equal to Zero,
  
  dec     R1                    ;  It is, Decrement the Message Counter

  sjmp    SendMsg_Find

SendMsg_Output:                 ;  Now, Output the String to '\0'

  mov     A,R0
  add     A,#LOW(SendMsgTable - SendMsgOutputGet)

  movc A,@A+PC                  ;  Read the Table
SendMsgOutputGet:

  inc     R0                    ;  Point to the Next Character

  jz      SendMsg_End           ;  If at Zero, Stop the Reading/Displaying

  lcall   SendChar              ;  Output the Character

  sjmp    SendMsg_Output

SendMsg_End:                    ;  Message Output 

  ret

SendMsgTable:                   ;  Messages to be Displayed
  db      CR,LF,'AT89C2051 Emulator',CR,LF,0        ;  Message  0
  db      'Invalid Command',CR,LF,0                 ;  Message  1
  db      'Enter New Value ',0                      ;  Message  2
  db      'Invalid Address',CR,LF,0                 ;  Message  3
  db      'No Address Specified',CR,LF,0            ;  Message  4
  db      CR,LF,0                                   ;  Message  5
  db      'Scratchpad RAM:',CR,LF,0                 ;  Message  6
  db      ':  ',0                                   ;  Message  7
  db      'SP = ',0                                 ;  Message  8  Order  3
  db      'DPTR = ',0                               ;  Message  9  Order  4
  db      'PCON = ',0                               ;  Message 10  Order  6
  db      'TCON = ',0                               ;  Message 11  Order  8
  db      'TMOD = ',0                               ;  Message 12  Order  8
  db      'TH0/TL0 = ',0                            ;  Message 13  Order  9
  db      'TH1/TL1 = ',0                            ;  Message 14  Order 10
  db      'P1 = ',0                                 ;  Message 15  Order  7
  db      'P3 = ',0                                 ;  Message 16  Order  7
  db      'SCON = ',0                               ;  Message 17  Order 11
  db      'SBUF = ',0                               ;  Message 18  Order 11
  db      'IE = ',0                                 ;  Message 19  Order  5
  db      'IP = ',0                                 ;  Message 20  Order  5
  db      'PSW = ',0                                ;  Message 21  Order  2
  db      'B = ',0                                  ;  Message 22  Order  1
SendMsg2:                       ;  More Messages to Send

  mov     R1,A                  ;  Save the Message Number

  mov     R0,#0                 ;  Use R0 as the Offset Counter

SendMsg2_Find:                  ;  Look for the Appropriate Message

  mov     A,R1                  ;  Are we Now At Zero?
  jz      SendMsg2_Output       ;   If Count is Zero

  mov     A,#LOW(SendMsg2Table - SendMsg2FindGet)
  add     A,R0                  ;  Get Current Address

  movc A,@A+PC                  ;  Get the Character in the Displays
SendMsg2FindGet:

  inc     R0                    ;  Point to the Next Character

  jnz     SendMsg2_Find         ;  If NOT Equal to Zero,
  
  dec     R1                    ;  It is, Decrement the Message Counter

  sjmp    SendMsg2_Find

SendMsg2_Output:                ;  Now, Output the String to '\0'

  mov     A,R0
  add     A,#LOW(SendMsg2Table - SendMsg2OutputGet)

  movc A,@A+PC                  ;  Read the Table
SendMsg2OutputGet:

  inc     R0                    ;  Point to the Next Character

  jz      SendMsg2_End          ;  If at Zero, Stop the Reading/Displaying

  lcall   SendChar              ;  Output the Character

  sjmp    SendMsg2_Output

SendMsg2_End:                   ;  Message Output 

  ret

SendMsg2Table:                  ;  Messages to be Displayed
  db      'Start Text Download of Hex File',CR,LF,0  ;  Message  0
  db      'Download Timeout',CR,LF,0                 ;  Message  1
  db      'Invalid Hex File Format',CR,LF,0          ;  Message  2
  db      'Breakpoint ',0                            ;  Message  3
  db      ' at Address ',0                           ;  Message  4
  db      'No Breakpoints',CR,LF,0                   ;  Message  5
  db      'Insufficient Space between jmp and Destination',CR,LF,0
  db      'Insufficient Space between Breakpoints',CR,LF,0
  db      'No Breakpoints Available',CR,LF,0



InsLength:                      ;  Get the Length for Each Byte

  inc     A                     ;  Increment to get Table Offset
  jnz     GetInsLength          ;   Do we Have 0FFh?

  mov     A,#1                  ;  Yes, Return "1"

  ret

GetInsLength:

  movc A,@A+PC                  ;  Read the Table

  ret

InsLengthTable:                 ;  Length of Each First Byte Instruction
  db      1                     ;  000h - Nop   - One Byte Long
  db      2                     ;  001h - ajmp  - One Byte Long
  db      3                     ;  002h - ljmp  - One Byte Long
  db      1                     ;  003h - rr    - One Byte Long
  db      1                     ;  004h - inc   - One Byte Long
  db      2                     ;  005h - inc   - One Byte Long
  db      1                     ;  006h - inc   - One Byte Long
  db      1                     ;  007h - inc   - One Byte Long
  db      1                     ;  008h - inc   - One Byte Long
  db      1                     ;  009h - inc   - One Byte Long
  db      1                     ;  00Ah - inc   - One Byte Long
  db      1                     ;  00Bh - inc   - One Byte Long
  db      1                     ;  00Ch - inc   - One Byte Long
  db      1                     ;  00Dh - inc   - One Byte Long
  db      1                     ;  00Eh - inc   - One Byte Long
  db      1                     ;  00Fh - inc   - One Byte Long
  db      3                     ;  010h - jbc   - One Byte Long
  db      2                     ;  011h - acall - One Byte Long
  db      3                     ;  012h - lcall - One Byte Long
  db      1                     ;  013h - rrc   - One Byte Long
  db      1                     ;  014h - dec   - One Byte Long
  db      2                     ;  015h - dec   - One Byte Long
  db      1                     ;  016h - dec   - One Byte Long
  db      1                     ;  017h - dec   - One Byte Long
  db      1                     ;  018h - dec   - One Byte Long
  db      1                     ;  019h - dec   - One Byte Long
  db      1                     ;  01Ah - dec   - One Byte Long
  db      1                     ;  01Bh - dec   - One Byte Long
  db      1                     ;  01Ch - dec   - One Byte Long
  db      1                     ;  01Dh - dec   - One Byte Long
  db      1                     ;  01Eh - dec   - One Byte Long
  db      1                     ;  01Fh - dec   - One Byte Long
  db      3                     ;  020h - jb    - One Byte Long
  db      2                     ;  021h - ajmp  - One Byte Long
  db      1                     ;  022h - ret   - One Byte Long
  db      1                     ;  023h - rl    - One Byte Long
  db      2                     ;  024h - add   - One Byte Long
  db      2                     ;  025h - add   - One Byte Long
  db      1                     ;  026h - add   - One Byte Long
  db      1                     ;  027h - add   - One Byte Long
  db      1                     ;  028h - add   - One Byte Long
  db      1                     ;  029h - add   - One Byte Long
  db      1                     ;  02Ah - add   - One Byte Long
  db      1                     ;  02Bh - add   - One Byte Long
  db      1                     ;  02Ch - add   - One Byte Long
  db      1                     ;  02Dh - add   - One Byte Long
  db      1                     ;  02Eh - add   - One Byte Long
  db      1                     ;  02Fh - add   - One Byte Long
  db      3                     ;  030h - jnb   - One Byte Long
  db      2                     ;  031h - acall - One Byte Long
  db      1                     ;  032h - reti  - One Byte Long
  db      1                     ;  033h - rlc   - One Byte Long
  db      2                     ;  034h - addc  - One Byte Long
  db      2                     ;  035h - addc  - One Byte Long
  db      1                     ;  036h - addc  - One Byte Long
  db      1                     ;  037h - addc  - One Byte Long
  db      1                     ;  038h - addc  - One Byte Long
  db      1                     ;  039h - addc  - One Byte Long
  db      1                     ;  03Ah - addc  - One Byte Long
  db      1                     ;  03Bh - addc  - One Byte Long
  db      1                     ;  03Ch - addc  - One Byte Long
  db      1                     ;  03Dh - addc  - One Byte Long
  db      1                     ;  03Eh - addc  - One Byte Long
  db      1                     ;  03Fh - addc  - One Byte Long
  db      2                     ;  040h - jc    - One Byte Long
  db      2                     ;  041h - ajmp  - One Byte Long
  db      2                     ;  042h - orl   - One Byte Long
  db      3                     ;  043h - orl   - One Byte Long
  db      2                     ;  044h - orl   - One Byte Long
  db      2                     ;  045h - orl   - One Byte Long
  db      1                     ;  046h - orl   - One Byte Long
  db      1                     ;  047h - orl   - One Byte Long
  db      1                     ;  048h - orl   - One Byte Long
  db      1                     ;  049h - orl   - One Byte Long
  db      1                     ;  04Ah - orl   - One Byte Long
  db      1                     ;  04Bh - orl   - One Byte Long
  db      1                     ;  04Ch - orl   - One Byte Long
  db      1                     ;  04Dh - orl   - One Byte Long
  db      1                     ;  04Eh - orl   - One Byte Long
  db      1                     ;  04Fh - orl   - One Byte Long
  db      2                     ;  050h - jnc   - One Byte Long
  db      2                     ;  051h - acall - One Byte Long
  db      2                     ;  052h - anl   - One Byte Long
  db      3                     ;  053h - anl   - One Byte Long
  db      2                     ;  054h - anl   - One Byte Long
  db      2                     ;  055h - anl   - One Byte Long
  db      1                     ;  056h - anl   - One Byte Long
  db      1                     ;  057h - anl   - One Byte Long
  db      1                     ;  058h - anl   - One Byte Long
  db      1                     ;  059h - anl   - One Byte Long
  db      1                     ;  05Ah - anl   - One Byte Long
  db      1                     ;  05Bh - anl   - One Byte Long
  db      1                     ;  05Ch - anl   - One Byte Long
  db      1                     ;  05Dh - anl   - One Byte Long
  db      1                     ;  05Eh - anl   - One Byte Long
  db      1                     ;  05Fh - anl   - One Byte Long
  db      2                     ;  060h - jz    - One Byte Long
  db      2                     ;  061h - ajmp  - One Byte Long
  db      2                     ;  062h - xrl   - One Byte Long
  db      3                     ;  063h - xrl   - One Byte Long
  db      2                     ;  064h - xrl   - One Byte Long
  db      2                     ;  065h - xrl   - One Byte Long
  db      1                     ;  066h - xrl   - One Byte Long
  db      1                     ;  067h - xrl   - One Byte Long
  db      1                     ;  068h - xrl   - One Byte Long
  db      1                     ;  069h - xrl   - One Byte Long
  db      1                     ;  06Ah - xrl   - One Byte Long
  db      1                     ;  06Bh - xrl   - One Byte Long
  db      1                     ;  06Ch - xrl   - One Byte Long
  db      1                     ;  06Dh - xrl   - One Byte Long
  db      1                     ;  06Eh - xrl   - One Byte Long
  db      1                     ;  06Fh - xrl   - One Byte Long
  db      2                     ;  070h - jnz   - One Byte Long
  db      2                     ;  071h - acall - One Byte Long
  db      2                     ;  072h - orl   - One Byte Long
  db      1                     ;  073h - jmp   - One Byte Long
  db      2                     ;  074h - mov   - One Byte Long
  db      3                     ;  075h - mov   - One Byte Long
  db      2                     ;  076h - mov   - One Byte Long
  db      2                     ;  077h - mov   - One Byte Long
  db      2                     ;  078h - mov   - One Byte Long
  db      2                     ;  079h - mov   - One Byte Long
  db      2                     ;  07Ah - mov   - One Byte Long
  db      2                     ;  07Bh - mov   - One Byte Long
  db      2                     ;  07Ch - mov   - One Byte Long
  db      2                     ;  07Dh - mov   - One Byte Long
  db      2                     ;  07Eh - mov   - One Byte Long
  db      2                     ;  07Fh - mov   - One Byte Long
  db      2                     ;  080h - sjmp  - One Byte Long
  db      2                     ;  081h - ajmp  - One Byte Long
  db      2                     ;  082h - anl   - One Byte Long
  db      1                     ;  083h - movc  - One Byte Long
  db      1                     ;  084h - div   - One Byte Long
  db      3                     ;  085h - mov   - One Byte Long
  db      2                     ;  086h - mov   - One Byte Long
  db      2                     ;  087h - mov   - One Byte Long
  db      2                     ;  088h - mov   - One Byte Long
  db      2                     ;  089h - mov   - One Byte Long
  db      2                     ;  08Ah - mov   - One Byte Long
  db      2                     ;  08Bh - mov   - One Byte Long
  db      2                     ;  08Ch - mov   - One Byte Long
  db      2                     ;  08Dh - mov   - One Byte Long
  db      2                     ;  08Eh - mov   - One Byte Long
  db      2                     ;  08Fh - mov   - One Byte Long
  db      3                     ;  090h - mov   - One Byte Long
  db      2                     ;  091h - acall - One Byte Long
  db      2                     ;  092h - mov   - One Byte Long
  db      1                     ;  093h - movc  - One Byte Long
  db      2                     ;  094h - subb  - One Byte Long
  db      2                     ;  095h - subb  - One Byte Long
  db      1                     ;  096h - subb  - One Byte Long
  db      1                     ;  097h - subb  - One Byte Long
  db      1                     ;  098h - subb  - One Byte Long
  db      1                     ;  099h - subb  - One Byte Long
  db      1                     ;  09Ah - subb  - One Byte Long
  db      1                     ;  09Bh - subb  - One Byte Long
  db      1                     ;  09Ch - subb  - One Byte Long
  db      1                     ;  09Dh - subb  - One Byte Long
  db      1                     ;  09Eh - subb  - One Byte Long
  db      1                     ;  09Fh - subb  - One Byte Long
  db      2                     ;  0A0h - orl   - One Byte Long
  db      2                     ;  0A1h - ajmp  - One Byte Long
  db      2                     ;  0A2h - mov   - One Byte Long
  db      1                     ;  0A3h - inc   - One Byte Long
  db      1                     ;  0A4h - mul   - One Byte Long
  db      1                     ;  0A5h
  db      2                     ;  0A6h - mov   - One Byte Long
  db      2                     ;  0A7h - mov   - One Byte Long
  db      2                     ;  0A8h - mov   - One Byte Long
  db      2                     ;  0A9h - mov   - One Byte Long
  db      2                     ;  0AAh - mov   - One Byte Long
  db      2                     ;  0ABh - mov   - One Byte Long
  db      2                     ;  0ACh - mov   - One Byte Long
  db      2                     ;  0ADh - mov   - One Byte Long
  db      2                     ;  0AEh - mov   - One Byte Long
  db      2                     ;  0AFh - mov   - One Byte Long
  db      2                     ;  0B0h - anl   - One Byte Long
  db      2                     ;  0B1h - acall - One Byte Long
  db      2                     ;  0B2h - cpl   - One Byte Long
  db      1                     ;  0B3h - cpl   - One Byte Long
  db      3                     ;  0B4h - cjne  - One Byte Long
  db      3                     ;  0B5h - cjne  - One Byte Long
  db      3                     ;  0B6h - cjne  - One Byte Long
  db      3                     ;  0B7h - cjne  - One Byte Long
  db      3                     ;  0B8h - cjne  - One Byte Long
  db      3                     ;  0B9h - cjne  - One Byte Long
  db      3                     ;  0BAh - cjne  - One Byte Long
  db      3                     ;  0BBh - cjne  - One Byte Long
  db      3                     ;  0BCh - cjne  - One Byte Long
  db      3                     ;  0BDh - cjne  - One Byte Long
  db      3                     ;  0BEh - cjne  - One Byte Long
  db      3                     ;  0BFh - cjne  - One Byte Long
  db      2                     ;  0C0h - push  - One Byte Long
  db      2                     ;  0C1h - ajmp  - One Byte Long
  db      2                     ;  0C2h - clr   - One Byte Long
  db      1                     ;  0C3h - clr   - One Byte Long
  db      1                     ;  0C4h - swap  - One Byte Long
  db      2                     ;  0C5h - xch   - One Byte Long
  db      1                     ;  0C6h - xch   - One Byte Long
  db      1                     ;  0C7h - xch   - One Byte Long
  db      1                     ;  0C8h - xch   - One Byte Long
  db      1                     ;  0C9h - xch   - One Byte Long
  db      1                     ;  0CAh - xch   - One Byte Long
  db      1                     ;  0CBh - xch   - One Byte Long
  db      1                     ;  0CCh - xch   - One Byte Long
  db      1                     ;  0CDh - xch   - One Byte Long
  db      1                     ;  0CEh - xch   - One Byte Long
  db      1                     ;  0CFh - xch   - One Byte Long
  db      2                     ;  0D0h - pop   - One Byte Long
  db      2                     ;  0D1h - acall - One Byte Long
  db      2                     ;  0D2h - setb  - One Byte Long
  db      1                     ;  0D3h - setb  - One Byte Long
  db      1                     ;  0D4h - da    - One Byte Long
  db      3                     ;  0D5h - djnz  - One Byte Long
  db      1                     ;  0D6h - xchd  - One Byte Long
  db      1                     ;  0D7h - xchd  - One Byte Long
  db      2                     ;  0D8h - djnz  - One Byte Long
  db      2                     ;  0D9h - djnz  - One Byte Long
  db      2                     ;  0DAh - djnz  - One Byte Long
  db      2                     ;  0DBh - djnz  - One Byte Long
  db      2                     ;  0DCh - djnz  - One Byte Long
  db      2                     ;  0DDh - djnz  - One Byte Long
  db      2                     ;  0DEh - djnz  - One Byte Long
  db      2                     ;  0DFh - djnz  - One Byte Long
  db      1                     ;  0E0h - movx  - One Byte Long
  db      2                     ;  0E1h - ajmp  - One Byte Long
  db      1                     ;  0E2h - movx  - One Byte Long
  db      1                     ;  0E3h - movx  - One Byte Long
  db      1                     ;  0E4h - clr   - One Byte Long
  db      2                     ;  0E5h - mov   - One Byte Long
  db      1                     ;  0E6h - mov   - One Byte Long
  db      1                     ;  0E7h - mov   - One Byte Long
  db      1                     ;  0E8h - mov   - One Byte Long
  db      1                     ;  0E9h - mov   - One Byte Long
  db      1                     ;  0EAh - mov   - One Byte Long
  db      1                     ;  0EBh - mov   - One Byte Long
  db      1                     ;  0ECh - mov   - One Byte Long
  db      1                     ;  0EDh - mov   - One Byte Long
  db      1                     ;  0EEh - mov   - One Byte Long
  db      1                     ;  0EFh - mov   - One Byte Long
  db      1                     ;  0F0h - movx  - One Byte Long
  db      2                     ;  0F1h - acall - One Byte Long
  db      1                     ;  0F2h - movx  - One Byte Long
  db      1                     ;  0F3h - movx  - One Byte Long
  db      1                     ;  0F4h - cpl   - One Byte Long
  db      2                     ;  0F5h - mov   - One Byte Long
  db      1                     ;  0F6h - mov   - One Byte Long
  db      1                     ;  0F7h - mov   - One Byte Long
  db      1                     ;  0F8h - mov   - One Byte Long
  db      1                     ;  0F9h - mov   - One Byte Long
  db      1                     ;  0FAh - mov   - One Byte Long
  db      1                     ;  0FBh - mov   - One Byte Long
  db      1                     ;  0FCh - mov   - One Byte Long
  db      1                     ;  0FDh - mov   - One Byte Long
  db      1                     ;  0FEh - mov   - One Byte Long


InsType1:                       ;  Get the Label To Use with the Instruction

  inc     A                     ;  Increment to get Table Offset
  jnz     GetInsType1           ;   Do we Have 0FFh?

  mov     A,#21                 ;  Yes, Return "21" for "mov   " 

  ret

GetInsType1:

  movc A,@A+PC                  ;  Read the Table

  ret

InsType1Table:                  ;  Length of Each First Byte Instruction
  db      54                    ;  000h - Nop   
  db      41                    ;  001h - ajmp  
  db      42                    ;  002h - ljmp  
  db      17                    ;  003h - rr    
  db      5                     ;  004h - inc   
  db      5                     ;  005h - inc   
  db      5                     ;  006h - inc   
  db      5                     ;  007h - inc   
  db      5                     ;  008h - inc  
  db      5                     ;  009h - inc  
  db      5                     ;  00Ah - inc  
  db      5                     ;  00Bh - inc  
  db      5                     ;  00Ch - inc  
  db      5                     ;  00Dh - inc  
  db      5                     ;  00Eh - inc  
  db      5                     ;  00Fh - inc  
  db      51                    ;  010h - jbc   
  db      37                    ;  011h - acall 
  db      38                    ;  012h - lcall 
  db      18                    ;  013h - rrc   
  db      6                     ;  014h - dec 
  db      6                     ;  015h - dec 
  db      6                     ;  016h - dec 
  db      6                     ;  017h - dec 
  db      6                     ;  018h - dec 
  db      6                     ;  019h - dec 
  db      6                     ;  01Ah - dec 
  db      6                     ;  01Bh - dec 
  db      6                     ;  01Ch - dec 
  db      6                     ;  01Dh - dec 
  db      6                     ;  01Eh - dec 
  db      6                     ;  01Fh - dec 
  db      49                    ;  020h - jb  
  db      41                    ;  021h - ajmp  
  db      39                    ;  022h - ret 
  db      15                    ;  023h - rl  
  db      1                     ;  024h - add 
  db      1                     ;  025h - add 
  db      1                     ;  026h - add 
  db      1                     ;  027h - add 
  db      1                     ;  028h - add 
  db      1                     ;  029h - add 
  db      1                     ;  02Ah - add 
  db      1                     ;  02Bh - add 
  db      1                     ;  02Ch - add 
  db      1                     ;  02Dh - add 
  db      1                     ;  02Eh - add 
  db      1                     ;  02Fh - add 
  db      50                    ;  030h - jnb 
  db      37                    ;  031h - acall
  db      40                    ;  032h - reti
  db      16                    ;  033h - rlc 
  db      2                     ;  034h - addc 
  db      2                     ;  035h - addc 
  db      2                     ;  036h - addc 
  db      2                     ;  037h - addc 
  db      2                     ;  038h - addc 
  db      2                     ;  039h - addc 
  db      2                     ;  03Ah - addc 
  db      2                     ;  03Bh - addc 
  db      2                     ;  03Ch - addc 
  db      2                     ;  03Dh - addc 
  db      2                     ;  03Eh - addc 
  db      2                     ;  03Fh - addc 
  db      47                    ;  040h - jc  
  db      41                    ;  041h - ajmp
  db      4                     ;  042h - orl 
  db      4                     ;  043h - orl 
  db      11                    ;  044h - orl 
  db      11                    ;  045h - orl 
  db      11                    ;  046h - orl 
  db      11                    ;  047h - orl 
  db      11                    ;  048h - orl 
  db      11                    ;  049h - orl 
  db      11                    ;  04Ah - orl 
  db      11                    ;  04Bh - orl 
  db      11                    ;  04Ch - orl 
  db      11                    ;  04Dh - orl 
  db      11                    ;  04Eh - orl 
  db      11                    ;  04Fh - orl 
  db      48                    ;  050h - jnc 
  db      37                    ;  051h - acall
  db      55                    ;  052h - anl 
  db      55                    ;  053h - anl 
  db      10                    ;  054h - anl 
  db      10                    ;  055h - anl 
  db      10                    ;  056h - anl 
  db      10                    ;  057h - anl 
  db      10                    ;  058h - anl 
  db      10                    ;  059h - anl 
  db      10                    ;  05Ah - anl 
  db      10                    ;  05Bh - anl 
  db      10                    ;  05Ch - anl 
  db      10                    ;  05Dh - anl 
  db      10                    ;  05Eh - anl 
  db      10                    ;  05Fh - anl 
  db      45                    ;  060h - jz  
  db      41                    ;  061h - ajmp 
  db      56                    ;  062h - xrl 
  db      56                    ;  063h - xrl 
  db      12                    ;  064h - xrl 
  db      12                    ;  065h - xrl 
  db      12                    ;  066h - xrl 
  db      12                    ;  067h - xrl 
  db      12                    ;  068h - xrl 
  db      12                    ;  069h - xrl 
  db      12                    ;  06Ah - xrl 
  db      12                    ;  06Bh - xrl 
  db      12                    ;  06Ch - xrl 
  db      12                    ;  06Dh - xrl 
  db      12                    ;  06Eh - xrl 
  db      12                    ;  06Fh - xrl 
  db      46                    ;  070h - jnz 
  db      37                    ;  071h - acall
  db      36                    ;  072h - orl 
  db      44                    ;  073h - jmp 
  db      20                    ;  074h - mov 
  db      21                    ;  075h - mov 
  db      21                    ;  076h - mov 
  db      21                    ;  077h - mov 
  db      21                    ;  078h - mov 
  db      21                    ;  079h - mov 
  db      21                    ;  07Ah - mov 
  db      21                    ;  07Bh - mov 
  db      21                    ;  07Ch - mov 
  db      21                    ;  07Dh - mov 
  db      21                    ;  07Eh - mov 
  db      21                    ;  07Fh - mov 
  db      43                    ;  080h - sjmp 
  db      41                    ;  081h - ajmp 
  db      35                    ;  082h - anl 
  db      22                    ;  083h - movc
  db      8                     ;  084h - div 
  db      21                    ;  085h - mov 
  db      21                    ;  086h - mov 
  db      21                    ;  087h - mov 
  db      21                    ;  088h - mov 
  db      21                    ;  089h - mov 
  db      21                    ;  08Ah - mov 
  db      21                    ;  08Bh - mov 
  db      21                    ;  08Ch - mov 
  db      21                    ;  08Dh - mov 
  db      21                    ;  08Eh - mov 
  db      21                    ;  08Fh - mov 
  db      21                    ;  090h - mov
  db      37                    ;  091h - acall
  db      21                    ;  092h - mov 
  db      22                    ;  093h - movc
  db      3                     ;  094h - subb 
  db      3                     ;  095h - subb 
  db      3                     ;  096h - subb 
  db      3                     ;  097h - subb 
  db      3                     ;  098h - subb 
  db      3                     ;  099h - subb 
  db      3                     ;  09Ah - subb 
  db      3                     ;  09Bh - subb 
  db      3                     ;  09Ch - subb 
  db      3                     ;  09Dh - subb 
  db      3                     ;  09Eh - subb 
  db      3                     ;  09Fh - subb 
  db      36                    ;  0A0h - orl 
  db      41                    ;  0A1h - ajmp
  db      21                    ;  0A2h - mov 
  db      5                     ;  0A3h - inc 
  db      7                     ;  0A4h - mul 
  db      1                     ;  0A5h
  db      21                    ;  0A6h - mov 
  db      21                    ;  0A7h - mov 
  db      21                    ;  0A8h - mov 
  db      21                    ;  0A9h - mov 
  db      21                    ;  0AAh - mov 
  db      21                    ;  0ABh - mov 
  db      21                    ;  0ACh - mov 
  db      21                    ;  0ADh - mov 
  db      21                    ;  0AEh - mov 
  db      21                    ;  0AFh - mov 
  db      35                    ;  0B0h - anl 
  db      37                    ;  0B1h - acall
  db      34                    ;  0B2h - cpl  
  db      33                    ;  0B3h - cpl  
  db      52                    ;  0B4h - cjne 
  db      52                    ;  0B5h - cjne 
  db      52                    ;  0B6h - cjne 
  db      52                    ;  0B7h - cjne 
  db      52                    ;  0B8h - cjne 
  db      52                    ;  0B9h - cjne 
  db      52                    ;  0BAh - cjne 
  db      52                    ;  0BBh - cjne 
  db      52                    ;  0BCh - cjne 
  db      52                    ;  0BDh - cjne 
  db      52                    ;  0BEh - cjne 
  db      52                    ;  0BFh - cjne 
  db      25                    ;  0C0h - push
  db      41                    ;  0C1h - ajmp
  db      30                    ;  0C2h - clr 
  db      29                    ;  0C3h - clr 
  db      19                    ;  0C4h - swap 
  db      27                    ;  0C5h - xch 
  db      27                    ;  0C6h - xch 
  db      27                    ;  0C7h - xch 
  db      27                    ;  0C8h - xch 
  db      27                    ;  0C9h - xch 
  db      27                    ;  0CAh - xch 
  db      27                    ;  0CBh - xch 
  db      27                    ;  0CCh - xch 
  db      27                    ;  0CDh - xch 
  db      27                    ;  0CEh - xch 
  db      27                    ;  0CFh - xch 
  db      26                    ;  0D0h - pop 
  db      37                    ;  0D1h - acall
  db      32                    ;  0D2h - setb 
  db      31                    ;  0D3h - setb 
  db      9                     ;  0D4h - da   
  db      53                    ;  0D5h - djnz 
  db      28                    ;  0D6h - xchd 
  db      28                    ;  0D7h - xchd 
  db      53                    ;  0D8h - djnz 
  db      53                    ;  0D9h - djnz 
  db      53                    ;  0DAh - djnz 
  db      53                    ;  0DBh - djnz 
  db      53                    ;  0DCh - djnz 
  db      53                    ;  0DDh - djnz 
  db      53                    ;  0DEh - djnz 
  db      53                    ;  0DFh - djnz 
  db      23                    ;  0E0h - movx 
  db      41                    ;  0E1h - ajmp 
  db      23                    ;  0E2h - movx 
  db      23                    ;  0E3h - movx 
  db      13                    ;  0E4h - clr 
  db      20                    ;  0E5h - mov 
  db      20                    ;  0E6h - mov 
  db      20                    ;  0E7h - mov 
  db      20                    ;  0E8h - mov 
  db      20                    ;  0E9h - mov 
  db      20                    ;  0EAh - mov 
  db      20                    ;  0EBh - mov 
  db      20                    ;  0ECh - mov 
  db      20                    ;  0EDh - mov 
  db      20                    ;  0EEh - mov 
  db      20                    ;  0EFh - mov 
  db      24                    ;  0F0h - movx 
  db      37                    ;  0F1h - acall
  db      24                    ;  0F2h - movx 
  db      24                    ;  0F3h - movx 
  db      14                    ;  0F4h - cpl 
  db      21                    ;  0F5h - mov 
  db      21                    ;  0F6h - mov 
  db      21                    ;  0F7h - mov 
  db      21                    ;  0F8h - mov 
  db      21                    ;  0F9h - mov 
  db      21                    ;  0FAh - mov 
  db      21                    ;  0FBh - mov 
  db      21                    ;  0FCh - mov 
  db      21                    ;  0FDh - mov 
  db      21                    ;  0FEh - mov 


InsLabel:                       ;  Send Message of the Instruction Type

  mov     R1,A                  ;  Save the Message Number
  dec     R1                    ;  Numbers Above are "1" Based

  mov     R0,#0                 ;  Use R0 as the Offset Counter

  clr     C                     ;  Now, See Where the Messages Fall
  subb    A,#28                 ;   Split up the Values
  jc      InsLabel_Find

  ljmp    InsLabel1             ;  Jump to the Code that Displays the Messages

InsLabel_Find:                  ;  Look for the Appropriate Message

  mov     A,R1                  ;  Are we Now At Zero?
  jz      InsLabel_Output       ;   If Count is Zero

  mov     A,#LOW(InsLabelTable - InsLabelFindGet)
  add     A,R0                  ;  Get Current Address

  movc A,@A+PC                  ;  Get the Character in the Displays
InsLabelFindGet:

  inc     R0                    ;  Point to the Next Character

  jnz     InsLabel_Find         ;  If NOT Equal to Zero,
  
  dec     R1                    ;  It is, Decrement the Message Counter

  sjmp    InsLabel_Find

InsLabel_Output:                ;  Now, Output the String to '\0'

  mov     A,R0
  add     A,#LOW(InsLabelTable - InsLabelOutputGet)

  movc A,@A+PC                  ;  Read the Table
InsLabelOutputGet:

  inc     R0                    ;  Point to the Next Character

  jz      InsLabel_End          ;  If at Zero, Stop the Reading/Displaying

  lcall   SendChar              ;  Output the Character

  sjmp    InsLabel_Output

InsLabel_End:                   ;  Message Output 

  ret

InsLabelTable:                   ;  Messages to be Displayed
  db      'add   A,',0           ;  Message  1
  db      'addc  A,',0           ;  Message  2
  db      'subb  A,',0           ;  Message  3
  db      'orl   ',0             ;  Message  4
  db      'inc   ',0             ;  Message  5
  db      'dec   ',0             ;  Message  6
  db      'mul   AB',0           ;  Message  7
  db      'div   AB',0           ;  Message  8
  db      'da    A',0            ;  Message  9
  db      'anl   A,',0           ;  Message 10
  db      'orl   A,',0           ;  Message 11
  db      'xrl   A,',0           ;  Message 12
  db      'clr   A',0            ;  Message 13
  db      'cpl   A',0            ;  Message 14
  db      'rl    A',0            ;  Message 15
  db      'rlc   A',0            ;  Message 16
  db      'rr    A',0            ;  Message 17
  db      'rrc   A',0            ;  Message 18
  db      'swap  A',0            ;  Message 19
  db      'mov   A,',0           ;  Message 20
  db      'mov   ',0             ;  Message 21
  db      'movc  A,',0           ;  Message 22
  db      'movx  A,',0           ;  Message 23
  db      'movx  ',0             ;  Message 24
  db      'push  ',0             ;  Message 25
  db      'pop   ',0             ;  Message 26
  db      'xch   ',0             ;  Message 27
  db      'xchd  ',0             ;  Message 28
InsLabel1:                      ;  Send Message of the Instruction Type

  mov     A,R1                  ;  Make the Message Number Correct for the Table Below
  clr     C
  subb    A,#27
  mov     R1,A
  dec     R1                    ;  Numbers Above are "1" Based

  mov     R0,#0                 ;  Use R0 as the Offset Counter

InsLabel1Find:                  ;  Look for the Appropriate Message

  mov     A,R1                  ;  Are we Now At Zero?
  jz      InsLabel1_Output      ;   If Count is Zero

  mov     A,#LOW(InsLabel1Table - InsLabel1FindGet)
  add     A,R0                  ;  Get Current Address

  movc A,@A+PC                  ;  Get the Character in the Displays
InsLabel1FindGet:

  inc     R0                    ;  Point to the Next Character

  jnz     InsLabel1Find         ;  If NOT Equal to Zero,
  
  dec     R1                    ;  It is, Decrement the Message Counter

  sjmp    InsLabel1Find

InsLabel1_Output:               ;  Now, Output the String to '\0'

  mov     A,R0
  add     A,#LOW(InsLabel1Table - InsLabel1OutputGet)

  movc A,@A+PC                  ;  Read the Table
InsLabel1OutputGet:

  inc     R0                    ;  Point to the Next Character

  jz      InsLabel1_End         ;  If at Zero, Stop the Reading/Displaying

  lcall   SendChar              ;  Output the Character

  sjmp    InsLabel1_Output

InsLabel1_End:                  ;  Message Output 

  ret

InsLabel1Table:                  ;  Messages to be Displayed
  db      'clr   C',0            ;  Message 29
  db      'clr   ',0             ;  Message 30
  db      'setb  C',0            ;  Message 31
  db      'setb  ',0             ;  Message 32
  db      'cpl   C',0            ;  Message 33
  db      'cpl   ',0             ;  Message 34
  db      'anl   C,',0           ;  Message 35
  db      'orl   C,',0           ;  Message 36
  db      'acall ',0             ;  Message 37
  db      'lcall ',0             ;  Message 38
  db      'ret',0                ;  Message 39
  db      'reti',0               ;  Message 40
  db      'ajmp  ',0             ;  Message 41
  db      'ljmp  ',0             ;  Message 42
  db      'sjmp  ',0             ;  Message 43
  db      'jmp   ',0             ;  Message 44
  db      'jz    ',0             ;  Message 45
  db      'jnz   ',0             ;  Message 46
  db      'jc    ',0             ;  Message 47
  db      'jnc   ',0             ;  Message 48
  db      'jb    ',0             ;  Message 49
  db      'jnb   ',0             ;  Message 50
  db      'jbc   ',0             ;  Message 51
  db      'cjne  ',0             ;  Message 52
  db      'djnz  ',0             ;  Message 53
  db      'nop',0                ;  Message 54
  db      'anl   ',0             ;  Message 55
  db      'xrl   ',0             ;  Message 56


InsType2:                       ;  Get the Label To Use with the Instruction

  inc     A                     ;  Increment to get Table Offset
  jnz     GetInsType2           ;   Do we Have 0FFh?

  mov     A,#10                 ;  Yes, Return "10" for "mov   Rn,A" 

  ret

GetInsType2:

  movc A,@A+PC                  ;  Read the Table

  ret

InsType2Table:                  ;  Length of Each First Byte Instruction
  db      6                     ;  000h - Nop   
  db      28                    ;  001h - ajmp  
  db      29                    ;  002h - ljmp  
  db      6                     ;  003h - rr    
  db      5                     ;  004h - inc   
  db      2                     ;  005h - inc   
  db      3                     ;  006h - inc   
  db      3                     ;  007h - inc   
  db      1                     ;  008h - inc  
  db      1                     ;  009h - inc  
  db      1                     ;  00Ah - inc  
  db      1                     ;  00Bh - inc  
  db      1                     ;  00Ch - inc  
  db      1                     ;  00Dh - inc  
  db      1                     ;  00Eh - inc  
  db      1                     ;  00Fh - inc  
  db      31                    ;  010h - jbc   
  db      28                    ;  011h - acall 
  db      29                    ;  012h - lcall 
  db      6                     ;  013h - rrc   
  db      5                     ;  014h - dec 
  db      2                     ;  015h - dec 
  db      3                     ;  016h - dec 
  db      3                     ;  017h - dec 
  db      1                     ;  018h - dec 
  db      1                     ;  019h - dec 
  db      1                     ;  01Ah - dec 
  db      1                     ;  01Bh - dec 
  db      1                     ;  01Ch - dec 
  db      1                     ;  01Dh - dec 
  db      1                     ;  01Eh - dec 
  db      1                     ;  01Fh - dec 
  db      31                    ;  020h - jb  
  db      28                    ;  021h - ajmp  
  db      39                    ;  022h - ret 
  db      6                     ;  023h - rl  
  db      4                     ;  024h - add 
  db      2                     ;  025h - add 
  db      3                     ;  026h - add 
  db      3                     ;  027h - add 
  db      1                     ;  028h - add 
  db      1                     ;  029h - add 
  db      1                     ;  02Ah - add 
  db      1                     ;  02Bh - add 
  db      1                     ;  02Ch - add 
  db      1                     ;  02Dh - add 
  db      1                     ;  02Eh - add 
  db      1                     ;  02Fh - add 
  db      31                    ;  030h - jnb 
  db      28                    ;  031h - acall
  db      6                     ;  032h - reti
  db      6                     ;  033h - rlc 
  db      4                     ;  034h - addc 
  db      2                     ;  035h - addc 
  db      3                     ;  036h - addc 
  db      3                     ;  037h - addc 
  db      1                     ;  038h - addc 
  db      1                     ;  039h - addc 
  db      1                     ;  03Ah - addc 
  db      1                     ;  03Bh - addc 
  db      1                     ;  03Ch - addc 
  db      1                     ;  03Dh - addc 
  db      1                     ;  03Eh - addc 
  db      1                     ;  03Fh - addc 
  db      30                    ;  040h - jc  
  db      28                    ;  041h - ajmp
  db      8                     ;  042h - orl 
  db      9                     ;  043h - orl 
  db      4                     ;  044h - orl 
  db      2                     ;  045h - orl 
  db      3                     ;  046h - orl 
  db      3                     ;  047h - orl 
  db      1                     ;  048h - orl 
  db      1                     ;  049h - orl 
  db      1                     ;  04Ah - orl 
  db      1                     ;  04Bh - orl 
  db      1                     ;  04Ch - orl 
  db      1                     ;  04Dh - orl 
  db      1                     ;  04Eh - orl 
  db      1                     ;  04Fh - orl 
  db      30                    ;  050h - jnc 
  db      28                    ;  051h - acall
  db      8                     ;  052h - anl 
  db      9                     ;  053h - anl 
  db      4                     ;  054h - anl 
  db      2                     ;  055h - anl 
  db      3                     ;  056h - anl 
  db      3                     ;  057h - anl 
  db      1                     ;  058h - anl 
  db      1                     ;  059h - anl 
  db      1                     ;  05Ah - anl 
  db      1                     ;  05Bh - anl 
  db      1                     ;  05Ch - anl 
  db      1                     ;  05Dh - anl 
  db      1                     ;  05Eh - anl 
  db      1                     ;  05Fh - anl 
  db      30                    ;  060h - jz  
  db      28                    ;  061h - ajmp 
  db      8                     ;  062h - xrl 
  db      9                     ;  063h - xrl 
  db      4                     ;  064h - xrl 
  db      2                     ;  065h - xrl 
  db      3                     ;  066h - xrl 
  db      3                     ;  067h - xrl 
  db      1                     ;  068h - xrl 
  db      1                     ;  069h - xrl 
  db      1                     ;  06Ah - xrl 
  db      1                     ;  06Bh - xrl 
  db      1                     ;  06Ch - xrl 
  db      1                     ;  06Dh - xrl 
  db      1                     ;  06Eh - xrl 
  db      1                     ;  06Fh - xrl 
  db      30                    ;  070h - jnz 
  db      28                    ;  071h - acall
  db      2                     ;  072h - orl 
  db      38                    ;  073h - jmp 
  db      4                     ;  074h - mov 
  db      9                     ;  075h - mov 
  db      18                    ;  076h - mov 
  db      18                    ;  077h - mov 
  db      12                    ;  078h - mov 
  db      12                    ;  079h - mov 
  db      12                    ;  07Ah - mov 
  db      12                    ;  07Bh - mov 
  db      12                    ;  07Ch - mov 
  db      12                    ;  07Dh - mov 
  db      12                    ;  07Eh - mov 
  db      12                    ;  07Fh - mov 
  db      30                    ;  080h - sjmp 
  db      28                    ;  081h - ajmp 
  db      2                     ;  082h - anl 
  db      21                    ;  083h - movc
  db      6                     ;  084h - div 
  db      14                    ;  085h - mov 
  db      15                    ;  086h - mov 
  db      15                    ;  087h - mov 
  db      13                    ;  088h - mov 
  db      13                    ;  089h - mov 
  db      13                    ;  08Ah - mov 
  db      13                    ;  08Bh - mov 
  db      13                    ;  08Ch - mov 
  db      13                    ;  08Dh - mov 
  db      13                    ;  08Eh - mov 
  db      13                    ;  08Fh - mov 
  db      19                    ;  090h - mov
  db      28                    ;  091h - acall
  db      27                    ;  092h - mov 
  db      20                    ;  093h - movc
  db      4                     ;  094h - subb 
  db      2                     ;  095h - subb 
  db      3                     ;  096h - subb 
  db      3                     ;  097h - subb 
  db      1                     ;  098h - subb 
  db      1                     ;  099h - subb 
  db      1                     ;  09Ah - subb 
  db      1                     ;  09Bh - subb 
  db      1                     ;  09Ch - subb 
  db      1                     ;  09Dh - subb 
  db      1                     ;  09Eh - subb 
  db      1                     ;  09Fh - subb 
  db      25                    ;  0A0h - orl 
  db      28                    ;  0A1h - ajmp
  db      26                    ;  0A2h - mov 
  db      7                     ;  0A3h - inc 
  db      6                     ;  0A4h - mul 
  db      1                     ;  0A5h
  db      17                    ;  0A6h - mov 
  db      17                    ;  0A7h - mov 
  db      11                    ;  0A8h - mov 
  db      11                    ;  0A9h - mov 
  db      11                    ;  0AAh - mov 
  db      11                    ;  0ABh - mov 
  db      11                    ;  0ACh - mov 
  db      11                    ;  0ADh - mov 
  db      11                    ;  0AEh - mov 
  db      11                    ;  0AFh - mov 
  db      25                    ;  0B0h - anl 
  db      28                    ;  0B1h - acall
  db      2                     ;  0B2h - cpl  
  db      6                     ;  0B3h - cpl  
  db      33                    ;  0B4h - cjne 
  db      32                    ;  0B5h - cjne 
  db      35                    ;  0B6h - cjne 
  db      35                    ;  0B7h - cjne 
  db      34                    ;  0B8h - cjne 
  db      34                    ;  0B9h - cjne 
  db      34                    ;  0BAh - cjne 
  db      34                    ;  0BBh - cjne 
  db      34                    ;  0BCh - cjne 
  db      34                    ;  0BDh - cjne 
  db      34                    ;  0BEh - cjne 
  db      34                    ;  0BFh - cjne 
  db      2                     ;  0C0h - push
  db      28                    ;  0C1h - ajmp
  db      2                     ;  0C2h - clr 
  db      6                     ;  0C3h - clr 
  db      6                     ;  0C4h - swap 
  db      2                     ;  0C5h - xch 
  db      3                     ;  0C6h - xch 
  db      3                     ;  0C7h - xch 
  db      1                     ;  0C8h - xch 
  db      1                     ;  0C9h - xch 
  db      1                     ;  0CAh - xch 
  db      1                     ;  0CBh - xch 
  db      1                     ;  0CCh - xch 
  db      1                     ;  0CDh - xch 
  db      1                     ;  0CEh - xch 
  db      1                     ;  0CFh - xch 
  db      2                     ;  0D0h - pop 
  db      28                    ;  0D1h - acall
  db      2                     ;  0D2h - setb 
  db      6                     ;  0D3h - setb 
  db      6                     ;  0D4h - da   
  db      37                    ;  0D5h - djnz 
  db      3                     ;  0D6h - xchd 
  db      3                     ;  0D7h - xchd 
  db      36                    ;  0D8h - djnz 
  db      36                    ;  0D9h - djnz 
  db      36                    ;  0DAh - djnz 
  db      36                    ;  0DBh - djnz 
  db      36                    ;  0DCh - djnz 
  db      36                    ;  0DDh - djnz 
  db      36                    ;  0DEh - djnz 
  db      36                    ;  0DFh - djnz 
  db      22                    ;  0E0h - movx 
  db      28                    ;  0E1h - ajmp 
  db      3                     ;  0E2h - movx 
  db      3                     ;  0E3h - movx 
  db      6                     ;  0E4h - clr 
  db      2                     ;  0E5h - mov 
  db      3                     ;  0E6h - mov 
  db      3                     ;  0E7h - mov 
  db      1                     ;  0E8h - mov 
  db      1                     ;  0E9h - mov 
  db      1                     ;  0EAh - mov 
  db      1                     ;  0EBh - mov 
  db      1                     ;  0ECh - mov 
  db      1                     ;  0EDh - mov 
  db      1                     ;  0EEh - mov 
  db      1                     ;  0EFh - mov 
  db      24                    ;  0F0h - movx 
  db      28                    ;  0F1h - acall
  db      23                    ;  0F2h - movx 
  db      23                    ;  0F3h - movx 
  db      6                     ;  0F4h - cpl 
  db      8                     ;  0F5h - mov 
  db      16                    ;  0F6h - mov 
  db      16                    ;  0F7h - mov 
  db      10                    ;  0F8h - mov 
  db      10                    ;  0F9h - mov 
  db      10                    ;  0FAh - mov 
  db      10                    ;  0FBh - mov 
  db      10                    ;  0FCh - mov 
  db      10                    ;  0FDh - mov 
  db      10                    ;  0FEh - mov 


InsDisplay:                     ;  Display the Instruction in R4/DPH/DPL

  mov     A,R4                  ;  Now, Display the Instruction as Hex Again
  lcall   InsType1              ;  Get the Character to display
  lcall   InsLabel              ;  Now, Display the Label

  mov     A,R4                  ;  Now, Display Parameters According to Instruction Type
  lcall   InsType2
  dec     A
  jnz     HUA_Ins2              ;  If "A" == 1, Put in "Rn"
  
  mov     A,#'R'
  lcall   SendChar

  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  ret

HUA_Ins2:                       ;  Did "A" == 2?
  dec     A
  jnz     HUA_Ins3              ;  Put in 2nd Byte

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins3:                       ;  Dis "A" == 3?
  dec     A
  jnz     HUA_Ins4

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  ret

HUA_Ins4:

  dec     A
  jnz     HUA_Ins5

  mov     A,#'#'
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins5:

  dec     A
  jnz     HUA_Ins6

  mov     A,#'A'
  lcall   SendChar

  ret

HUA_Ins6:

  dec     A
  jnz     HUA_Ins7

  ret   

HUA_Ins7:

  dec     A
  jnz     HUA_Ins8

  mov     A,DPL1
  lcall   SendHex

HUA_InsEnd:

  mov     A,#','
  lcall   SendChar
  mov     A,#'A'
  lcall   SendChar

  ret

HUA_Ins8:

  dec     A
  jnz     HUA_Ins9

  mov     A,DPL1
  lcall   SendHex

  sjmp    HUA_InsEnd

HUA_Ins9:

  dec     A
  jnz     HUA_Ins10

  mov     A,DPH1
  lcall   SendHex
  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar
  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins10:

  dec     A
  jnz     HUA_Ins11

  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  sjmp    HUA_InsEnd

HUA_Ins11:

  dec     A
  jnz     HUA_Ins12

  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins12:

  dec     A
  jnz     HUA_Ins13

  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins13:

  dec     A
  jnz     HUA_Ins14

  mov     A,DPL1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  ret

HUA_Ins14:

  dec     A
  jnz     HUA_Ins15

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins15:

  dec     A
  jnz     HUA_Ins16

  mov     A,DPL1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar
  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  ret

HUA_Ins16:

  dec     A
  jnz     HUA_Ins17

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'A'
  lcall   SendChar

  ret

HUA_Ins17:

  dec     A
  jnz     HUA_Ins18

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins18:

  dec     A
  jnz     HUA_Ins19

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins19:

  dec     A
  jnz     HUA_Ins20

  mov     A,#'D'
  lcall   SendChar
  mov     A,#'P'
  lcall   SendChar
  mov     A,#'T'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar

  mov     A,DPH1
  lcall   SendHex
  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins20:

  dec     A
  jnz     HUA_Ins21

  mov     A,#'A'
  lcall   SendChar
  mov     A,#','
  lcall   SendChar

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'A'
  lcall   SendChar
  mov     A,#'+'
  lcall   SendChar
  mov     A,#'D'
  lcall   SendChar
  mov     A,#'P'
  lcall   SendChar
  mov     A,#'T'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  ret

HUA_Ins21:

  dec     A
  jnz     HUA_Ins22

  mov     A,#'A'
  lcall   SendChar
  mov     A,#','
  lcall   SendChar

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'A'
  lcall   SendChar
  mov     A,#'+'
  lcall   SendChar
  mov     A,#'P'
  lcall   SendChar
  mov     A,#'C'
  lcall   SendChar

  ret

HUA_Ins22:

  dec     A
  jnz     HUA_Ins23

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'D'
  lcall   SendChar
  mov     A,#'P'
  lcall   SendChar
  mov     A,#'T'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  ret

HUA_Ins23:

  dec     A
  jnz     HUA_Ins24

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'A'
  lcall   SendChar

  ret

HUA_Ins24:

  dec     A
  jnz     HUA_Ins25

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'D'
  lcall   SendChar
  mov     A,#'P'
  lcall   SendChar
  mov     A,#'T'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  ret

HUA_Ins25:

  dec     A
  jnz     HUA_Ins26

  mov     A,#'!'
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins26:

  dec     A
  jnz     HUA_Ins27

  mov     A,#'C'
  lcall   SendChar
  mov     A,#','
  lcall   SendChar

  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins27:

  dec     A
  jnz     HUA_Ins28

  mov     A,DPL1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar
  mov     A,#'C'
  lcall   SendChar

  ret

HUA_Ins28:

  dec     A
  jnz     HUA_Ins29

  mov     A,R4
  anl     A,#0E0h
  swap A
  rr A
  lcall   SendHex
  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins29:

  dec     A
  jnz     HUA_Ins30

  mov     A,DPH1
  lcall   SendHex
  mov     A,DPL1
  lcall   SendHex

  ret

HUA_Ins30:

  dec     A
  jnz     HUA_Ins31

  mov     A,DPL1
  lcall   SendDec

  ret

HUA_Ins31:

  dec     A
  jnz     HUA_Ins32

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     A,DPL1
  lcall   SendDec

  ret

HUA_Ins32:

  dec     A
  jnz     HUA_Ins33

  mov     A,#'A'
  lcall   SendChar
  mov     A,#','
  lcall   SendChar

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     a,DPL1
  lcall   SendDec

  ret

HUA_Ins33:

  dec     A
  jnz     HUA_Ins34

  mov     A,#'A'
  lcall   SendChar
  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     a,DPL1
  lcall   SendDec

  ret

HUA_Ins34:

  dec     A
  jnz     HUA_Ins35

  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     a,DPL1
  lcall   SendDec

  ret

HUA_Ins35:

  dec     A
  jnz     HUA_Ins36

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#1
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar
  mov     A,#'#'
  lcall   SendChar

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     a,DPL1
  lcall   SendDec

  ret

HUA_Ins36:

  dec     A
  jnz     HUA_Ins37

  mov     A,#'R'
  lcall   SendChar
  mov     A,R4
  anl     A,#7
  add     A,#'0'
  lcall   SendChar

  mov     A,#','
  lcall   SendChar

  mov     a,DPL1
  lcall   SendDec

  ret

HUA_Ins37:

  dec     A
  jnz     HUA_Ins38

  mov     A,DPH1
  lcall   SendHex

  mov     A,#','
  lcall   SendChar

  mov     a,DPL1
  lcall   SendDec

  ret

HUA_Ins38:

  mov     A,#'@'
  lcall   SendChar
  mov     A,#'A'
  lcall   SendChar
  mov     A,#'+'
  lcall   SendChar
  mov     A,#'D'
  lcall   SendChar
  mov     A,#'P'
  lcall   SendChar
  mov     A,#'T'
  lcall   SendChar
  mov     A,#'R'
  lcall   SendChar

  ret


EMUReset:                       ;  Reset the Emulated AT89C2051

  mov     R0,#ProgC             ;  Reset the Program Counter First
  mov     @R0,#0
  inc     R0
  mov     @R0,#0

  mov     R0,#(SavedRegs + 8)
  mov     @R0,#7                ;  Reset the Stack Pointer

  inc     R0
  mov     @R0,#0                ;  Reset DPTR
  inc     R0
  mov     @R0,#0

  inc     R0                    ;  Reset the Interrupt Controller
  mov     @R0,#0

  inc     R0                    ;  Reset PSW
  mov     @R0,#0

  inc     R0                    ;  Reset the Accumulator
  mov     @R0,#0

  inc     R0                    ;  Reset the "B" Register
  mov     @R0,#0

  mov     PCON,#0               ;  Reset the Other Registers

  mov     TCON,#0

  mov     TMOD,#0

  mov     TL0,#0
  mov     TH0,#0

  mov     TL1,#0
  mov     TH1,#0

  mov     P1,#0FFh
  mov     P3,#0FFh

  mov     SCON,#0

  mov     IP,#0

  ret
