//  PROG35D - Atmel AT89C2051 Programmer - Toggling the Prog Pin
//
//  This Program Simply Turns On the Device to be Programmed Power
//   Followed by the Reset Voltage Levels.  With Power On and Reset
//   valid, the Programming Bit Updates are Checked.  After
//   Programming Bits are Checked, then the Increment Function is
//   Checked.
//
//  PROG35D - Check the "PROG" Bit Level
//
//  Myke Predko
//
//  Started: 98.06.13
//
//  Update History:
//
//

//  Include Files
#include <stdio.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <dos.h>
#include <conio.h>


//  Defines


//  Global Variables
int  Value374;                  //  The Current '374 Value
char FileName[ 65 ];
int  Port = -1;                 //  Parallel Port Used
int  PortAddr;                  //   Address
long FileEnd;                   //  Size of the .hex file
char far * Program;             //  Pointer to the Hex File Buffer


//  Subroutines
StrobeDelay()                   //  Delay 50 usecs for the Stobe
{

int  Temp = 0;                  //  Wait For the Value to be Exceeded


  outp( 0x043, 0x0B6 );         //  Use Timer2 As the Delay

  outp( 0x042, 0x000 );         //  Load it with 0x00000
  outp( 0x042, 0x000 );

  outp( 0x061, inp( 0x061 ) | 1 );  //  Enable the Clock

  while ( Temp < -60 ) {        //  Wait 50 usecs
    Temp = inp( 0x042 );        //  Read the Timer Contents
    Temp = Temp + ( inp( 0x042 ) * 0x0100 );
  }  /*  endwhile  */

  outp( 0x061, inp( 0x061 ) & 0x0FE );  //  Disable the Clock

}  //  End StrobeDelay


Power( int Value )              // Turn On or Off the Power
{

  if ( Value == 1 )             //  Turn Power Onto the MCU?
    outp( PortAddr, Value374 = 0x000 );
  else                          //  else - Turn Off the Power
    outp( PortAddr, Value374 = 0x080 );

  outp( PortAddr+2, 0x00D );

  if ( Value == 1 )             //  Turning on Power, Strobe High
    outp( PortAddr+2, 0x004 );
  else
    outp( PortAddr+2, 0x005 );  //  Power Off, Strobe Low

}  //  End Power


TogglePROG()                    //  Toggle the "PROG" Bit
{

  outp( PortAddr+2, 0x005 );    //  Strobe Low

  StrobeDelay();

  outp( PortAddr+2, 0x004 );    //  Strobe Back to Low

}  //  End TogglePROG


Reset( int Value )              // Set the Correct Reset Value
{

  switch ( Value )
    {
      case 0:                   //  Turn OFF Reset
        outp( PortAddr, Value374 = Value374 & 0x09F );
        break;
      case 5:                   //  Reset to +5 Volts
        outp( PortAddr, Value374 = ( Value374 & 0x09F ) | 0x040 );
        break;
      case 12:                  //  Reset to +12 Volts
        outp( PortAddr, Value374 = ( Value374 & 0x09F ) | 0x020 );
        break;
    } /*  endswitch  */

  outp( PortAddr+2, 0x00D );
  outp( PortAddr+2, 0x004 );    //  Turning on Power, Strobe High

}  //  End Reset


ProgPin( int Value )            // Set the Correct Programming Pins
{                               //  Bit 0 of Value - P3.3 - PP Bit 4
                                //  Bit 1          - P3.4 - PP Bit 1
int  Temp = 0;                  //  Bit 2          - P3.5 - PP Bit 0
                                //  Bit 3          - P3.7 - PP Bit 2

  if (( Value & 1 ) != 0 )      //  Set the Bits to Write
    Temp = 0x010;
  if (( Value & 2 ) != 0 )
    Temp += 2;
  if (( Value & 4 ) != 0 )
    Temp += 1;
  if (( Value & 8 ) != 0 )
    Temp += 4;
  outp( PortAddr, Value374 = ( Value374 & 0x0E8 ) | Temp );

  outp( PortAddr+2, 0x00D );
  outp( PortAddr+2, 0x004 );    //  Turning on Power, Strobe High

}  //  End ProgPin


Increment()                     // Increment the XTAL1 Pin
{


  outp( PortAddr, Value374 | 8 );
  outp( PortAddr+2, 0x00D );    //  Increment Strobe High
  outp( PortAddr+2, 0x004 );

  outp( PortAddr, Value374 );   //  Increment Strobe Low
  outp( PortAddr+2, 0x00D );
  outp( PortAddr+2, 0x004 );

}  //  End Increment


MemFree()                       //  Free the Memory Used by "Program"
{

union REGS regs;
struct SREGS sregs;


  regs.h.ah = 0x049;
  sregs.es = (long) Program / 0x010000;
  intdosx( &regs, &regs, &sregs );

}  /*  End MemFree  */


TimeDlay()                      //  Timer Tick Delay
{

unsigned far * TimePtr;         //  Pointer to LSB Word of the Time

unsigned Original;              //  The Original Time

  (long) TimePtr = 0x00046CL;   //  LSWord of Time Value

  Original = *TimePtr + 2;

  if ( Original < 2 )           //  Worst Case, *TimePtr > Original
    while ( *TimePtr != 2 );    //   Try this for Waiting
  else
    while ( Original > *TimePtr );  //  Wait 55+ msec

}  //  TimeDlay


int cline( int argc,
  char *argv[],
  char *envp )
{

union REGS regs;

int  i, k;
long j;
int  handle;
int  retvalue = 2;              //  Return Value - Assume Good


  FileName[ 0 ] = '\0';         //  Mark the Start of the FileName

  if ( argc > 1 )               //  Only Handle if there are Args
    for ( i = 1; ( i < argc ) && ( retvalue != -1 ); i++ )
      switch ( argv[ i ][ 0 ])  //  Look at Each Character
        {
          case '1':             //  Handle the Port Specification
            if ( Port == -1 )
              {                 //  Specify the Port Value
                Port = 1;  PortAddr = 0x0378;
              }
            else
              {
                printf( "Multiple Port Specification\n" );
                printf( "Only ONE Port can be Defined\n" );
                retvalue = -1;
              }  /*  endif  */
            break;
          case '2':             //  Handle the Port Specification
            if ( Port == -1 )
              {                 //  Specify the Port Value
                Port = 2;  PortAddr = 0x0278;
              }
            else
              {
                printf( "Multiple Port Specification\n" );
                printf( "Only ONE Port can be Defined\n" );
                retvalue = -1;
              }  /*  endif  */
            break;
          default:;             //  Else, File Name Specified
            strncpy( &FileName[ 1 ], &argv[ i ][ 0 ],
                            FileName[ 0 ] = strlen( &argv[ i ][ 0 ]));
            for ( j = FileName[ 0 ] + 1; ( j > 1 ) &&
                                          ( FileName[ j ] != '\\' ) &&
                                      ( FileName[ j ] != '.' ); j-- );

            if ( FileName[ j ] != '.' )
              if ( FileName[ 0 ] > 59 )
                {
                  printf( "File name is TOO LARGE\n" );
                  retvalue = -1;
                }
              else
                {
                  strncpy( &FileName[ 1 + FileName[ 0 ]], ".hex", 4 );
                  FileName[ 0 ] += 4;
                }  /*  endif  */

            if ( retvalue != -1 )
              {
                FileName[ 1 + FileName[ 0 ]++ ] = '\0';
                if (( handle = open( &FileName[ 1 ],
                                        O_RDONLY | O_BINARY )) == -1 )
                  {
                    printf( "File \"%s\" was NOT FOUND\n",
                                                      &FileName[ 1 ]);
                    retvalue = -1;
                  }
                else            //  File was Opened, Okay, Read it
                  {
                    FileEnd = lseek( handle, 0L, SEEK_END );
                    if ( FileEnd == 0 )
                      {
                        printf( "File \"%s\" is EMPTY\n",
                                                      &FileName[ 1 ]);
                        retvalue = 1;
                      }
                    else        //  Get Memory & Store the .hex file
                      {

                        lseek( handle, 0L, SEEK_SET );

                        regs.h.ah = 0x048;
                        regs.x.bx = ( FileEnd + 0x010 )/ 0x010;
                        intdos( &regs, &regs );
                        (long) Program = regs.x.ax * 0x010000;

                        read( handle, Program, FileEnd );
                                //  Read in the .hex File
                        if ( Program[ FileEnd - 1 ] == 0x01A )
                          FileEnd--;
                        Program[ FileEnd++ ] = '\0';
                        if (( Program[ 0 ] != ':' ) ||
                                   ( strcmp( &Program[ FileEnd - 14 ],
                                           ":00000001FF\r\n" ) != 0 ))
                          {
                            printf( "File \"%s\" is NOT a Valid Hex File\n",
                                                      &FileName[ 1 ]);
                            retvalue = -1;
                            MemFree();  //  Free Up the Memory Used
                          }  /*  endif  */
                      }  /*  endif  */
                  }  /*  endif  */
              }  /*  endif  */
        } /*  endswitch  */

  if ( FileName[ 0 ] == '\0' )
    {
      printf( "NO Filename Specified\n" );
      retvalue = -1;
    }  /*  endif  */

  if ( Port == -1 )             //  No Port Specified
    {
      Port = 1;                 //  Default to "LPT1"
      PortAddr = 0x0378;
    }  /*  endif  */

  return( retvalue );

}  /*  End cline  */

int ReadHex( char far * Data )
{                               //  Convert two characters to Hex

int retvalue;


  if ( Data[ 0 ] >= 'A' )
    retvalue = ( Data[ 0 ] - 'A' + 10 ) * 0x010;
  else
    retvalue = ( Data[ 0 ] - '0' ) * 0x010;

  if ( Data[ 1 ] >= 'A' )
    retvalue += ( Data[ 1 ] - 'A' + 10 );
  else
    retvalue += ( Data[ 1 ] - '0' );

  return( retvalue );

}  //  End ReadHex


Prog()                          //  Now, Program the Part
{

int  retvalue = 0;
int  CurAddress = 0;            //  Current Address Being Written to
int  LineLength;                //  Number of Characters to Write To
int  LineAddress;

int  Temp;                      //  Data Read from the File
int  Data;                      //  Data to Program into the MCU

int  i = 0;
int  j, k;


  Power( 0 );                   //  Make Sure the Power is Off

  printf( "Turn off Programmer Power - Press any key when ready\n" );
  getch();

  printf( "Put AT89C2051 into Programmer - Press any key when ready\n" );
  getch();

  printf( "Turn ON Programmer Power - Press any key when ready\n" );
  getch();

  Power( 1 );

  TimeDlay();                   //  Wait for the Power ON to Stabilize

  outp( PortAddr, 0x040 );      //  Put Reset & Strobe High
  outp( PortAddr + 2, 0x00C );
  outp( PortAddr + 2, 0x004 );

  printf( "\nErasing Flash\n" );

  TimeDlay();                   //  Now, Erase the Flash

  outp( PortAddr, 0x021 );      //  Setup the Command with 12V
  outp( PortAddr + 2, 0x00C );
  outp( PortAddr + 2, 0x004 );

  TimeDlay();

  outp( PortAddr + 2, 0x005 );  //  Pulse the Strobe Low
  TimeDlay();
  outp( PortAddr + 2, 0x004 );

  TimeDlay();

  outp( PortAddr, 0x03C );      //  Now, Program the Byte
  outp( PortAddr + 2, 0x00C );
  outp( PortAddr + 2, 0x004 );

  printf( "\nProgramming" );

  while (( i++ < ( FileEnd - 14 )) && ( retvalue == 0 ))
    {                           //  Loop Through Each Line
      LineLength = ReadHex( &Program[ i ]);  i += 2;
      LineAddress = ( ReadHex( &Program[ i ]) * 0x0100 ) +
                                          ReadHex( &Program[ i + 2 ]);
      i += 6;                   //  Skip Over Address/Line Good

      for ( ; CurAddress < LineAddress; CurAddress++ )
        {                       //  Check to See If Increment Required

          outp( PortAddr, 0x03E );  //  Increment the Address
          outp( PortAddr + 2, 0x00C );
          outp( PortAddr + 2, 0x004 );

          TimeDlay();           //  Let Everything get Set up

          outp( PortAddr, 0x03C );  //  Now, Program the Byte
          outp( PortAddr + 2, 0x00C );
          outp( PortAddr + 2, 0x004 );

          TimeDlay();

        }  /*  endfor  */

      for ( j = 0; ( j < LineLength ) && ( retvalue == 0 ); j++ )
        {                       //  Now Put in the Data
          Temp = ReadHex( &Program[ i ]);  i += 2;  Data = 0;
          for ( k = 0; k < 8; k++ )
            {                   //  Reverse the Data Bits
              Data = ( Data << 1 ) + ( Temp & 1 );
              Temp = Temp >> 1;
            }  /*  endfor  */

          outp( PortAddr, Data );  //  Output the Data to P1
          outp( PortAddr + 2, 0x002 );

          TimeDlay();

          outp( PortAddr + 2, 0x003 );  //  Strobe Out the Data

          TimeDlay();

          outp( PortAddr + 2, 0x004 );  //  Restore the Output Port

          TimeDlay();


          outp( PortAddr, 0x054 );  //  Read Back What was Written
          outp( PortAddr + 2, 0x00C );
          outp( PortAddr + 2, 0x004 );

          TimeDlay();

          outp( PortAddr + 2, 0x0B6 );

          TimeDlay();

          Temp = inp( PortAddr );

          outp( PortAddr + 2, 0x004 );  //  Restore the Output State

          TimeDlay();

          outp( PortAddr, 0x03C );  //  Now, Program the Byte
          outp( PortAddr + 2, 0x00C );
          outp( PortAddr + 2, 0x004 );

          TimeDlay();           //  Let Everything get Set up

          outp( PortAddr, 0x03E );  //  Increment the Address
          outp( PortAddr + 2, 0x00C );
          outp( PortAddr + 2, 0x004 );

          TimeDlay();

          outp( PortAddr, 0x03C );  //  Now, Program the Byte
          outp( PortAddr + 2, 0x00C );
          outp( PortAddr + 2, 0x004 );

          TimeDlay();           //  Let Everything get Set up

          if ( Data != Temp )
            {                   //  Miscompare on Write
              printf( "\nData Miscompare at Address 0x0%4x\n",
                                                         CurAddress );
              printf( "0x0%2x Read Back, 0x0%2x Expected\n",
                                                         Temp, Data );
              retvalue = -1;
            }
          else                  //  Read Back was Okay, Indicate it
            {

              printf( "." );    //  Show the Progress
              CurAddress++;     //  Increment the Current Address
            }  /*  endif  */
        }  /*  endfor  */

      i += 4;                   //  Skip Last 4 Characters on Line

    }  /*  endwhile  */

  outp( PortAddr, 0x000 );      // Turning Off the Power
  outp( PortAddr+2, 0x00D );    //   - Turn Off the I/O First
  outp( PortAddr+2, 0x005 );

  Power( 0 );

  printf( "\n\nTurn off Programmer Power - Press any key when ready\n" );
  getch();

  printf( "Take AT89C2051 out of Programmer - Press any key when ready\n" );
  getch();

}  //  End Prog


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

int  retvalue;


  printf( "PROG35A\n" );
  printf( "Written by: Myke Predko\n\n" );



  if (( retvalue = cline( argc, argv, envp )) != -1 )
    {

      Power( 0 );               //  Initialize the '374 and Turn OFF
                                //   the Power
      Power( 1 );               //  Turn On the Power

      Reset( 5 );               //  Make Reset +5 Volts

      Reset( 12 );              //  Enable the +12 Volt Reset

      ProgPin( 1 );             //  Just Turn On Bit P3.3
      ProgPin( 2 );             //  Just Turn On Bit P3.4
      ProgPin( 4 );             //  Just Turn On Bit P3.5
      ProgPin( 8 );             //  Just Turn On Bit P3.7

      Increment();              //  Increment the PC
      Increment();

      TogglePROG();             //  Toggle the "PROG" Bit Low
      TogglePROG();

      Reset( 0 );               //  Turn off the Reset

      Power( 0 );               //  Turn off the Power

//    Prog();                   //  If File Name and Port Okay, Program

//    MemFree();                //  Free the Memory Created by "cline"

    }  /*  endif  */

}  // End Mainline
