//  PROG35G - Atmel AT89C2051 Programmer - Testing Programming
//
//  This Program Simply Turns On the Device to be Programmed Power
//   Followed by the Reset Voltage Levels.  With Power On and Reset
//   valid, the Programming Bit Updates are Checked.  After
//   Programming Bits are Checked, then the Increment Function is
//   Checked.
//
//  PROG35G - Start Programming
//
//  PROG35E - Output Data To the AT89Cx051
//
//  PROG35D - Check the "PROG" Bit Level
//
//  Myke Predko
//
//  Started: 98.06.13
//
//  Update History:
//
//

//  Include Files
#include <stdio.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <dos.h>
#include <conio.h>


//  Defines


//  Global Variables
int  Value374;                  //  The Current '374 Value
char FileName[ 65 ];
int  Port = -1;                 //  Parallel Port Used
int  PortAddr;                  //   Address
long FileEnd;                   //  Size of the .hex file
char far * Program;             //  Pointer to the Hex File Buffer
int  CurAddress = 0;            //  Current Address Being Written to


//  Subroutines
StrobeDelay()                   //  Delay 50 usecs for the Stobe
{

int  Temp = 0;                  //  Wait For the Value to be Exceeded


  outp( 0x043, 0x0B6 );         //  Use Timer2 As the Delay

  outp( 0x042, 0x000 );         //  Load it with 0x00000
  outp( 0x042, 0x000 );

  outp( 0x061, inp( 0x061 ) | 1 );  //  Enable the Clock

  while ( Temp < -60 ) {        //  Wait 50 usecs
    Temp = inp( 0x042 );        //  Read the Timer Contents
    Temp = Temp + ( inp( 0x042 ) * 0x0100 );
  }  /*  endwhile  */

  outp( 0x061, inp( 0x061 ) & 0x0FE );  //  Disable the Clock

}  //  End StrobeDelay


TimeDelay()                     //  Timer Tick Delay
{

unsigned far * TimePtr;         //  Pointer to LSB Word of the Time

unsigned Original;              //  The Original Time

  (long) TimePtr = 0x00046CL;   //  LSWord of Time Value

  Original = *TimePtr + 2;

  if ( Original < 2 )           //  Worst Case, *TimePtr > Original
    while ( *TimePtr != 2 );    //   Try this for Waiting
  else
    while ( Original > *TimePtr );  //  Wait 55+ msec

}  //  End TimeDelay


SendData( int Temp )            //  Output the Data to P1
{

int  i;
char Data = 0;                  //  Data to Output

  for ( i = 0; i < 8; i++ ) {   //  Reverse the Data Bits
    Data = ( Data << 1 ) + ( Temp & 1 );
    Temp = Temp >> 1;
  }

  outp( PortAddr, Data );
  outp( PortAddr + 2, 0x002 );

  StrobeDelay();

}  //  End SendData


char ReadData()                 //  Read the Data Back
{

int  i;
int  Data = 0;                  //  Data to Return
int  Temp;

  ProgPin( 0x00C );             //  Setup the Control Bits

  StrobeDelay();

  outp( PortAddr + 2, 0x0B6 );  //  Set the Port for Read

  TimeDelay();

  StrobeDelay();                //  Put in a 400 usec Setup for Data
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();
  StrobeDelay();

  Temp = inp( PortAddr );       //  Get the Read Value

  outp( PortAddr + 2, 0x004 );  //  Restore the Output State

  StrobeDelay();

  ProgPin( 0x00E );             //  Return to Programming Mode

  StrobeDelay();

  for ( i = 0; i < 8; i++ ) {   //  Reverse the Data Bits
    Data = ( Data << 1 ) + ( Temp & 1 );
    Temp = Temp >> 1;
  }

  return Data;                  //  Return the Read Value as 8 Bits

} //  End ReadData


PCIncrement()                   //  Increment the Program Counter
{

  outp( PortAddr, Value374 | 0x008 );
  outp( PortAddr + 2, 0x00C );  //  Increment the Address
  outp( PortAddr + 2, 0x004 );

  StrobeDelay();

  outp( PortAddr, Value374 );
  outp( PortAddr + 2, 0x00C );  //  Now, Return to Byte Program
  outp( PortAddr + 2, 0x004 );

  StrobeDelay();

  CurAddress++;                 //  Increment the Program Counter

}  //  End PCIncrement


Power( int Value )              // Turn On or Off the Power
{

  if ( Value == 1 )             //  Turn Power Onto the MCU?
    outp( PortAddr, Value374 = 0x000 );
  else                          //  else - Turn Off the Power
    outp( PortAddr, Value374 = 0x080 );

  outp( PortAddr+2, 0x00D );

  if ( Value == 1 )             //  Turning on Power, Strobe High
    outp( PortAddr+2, 0x004 );
  else
    outp( PortAddr+2, 0x005 );  //  Power Off, Strobe Low

  TimeDelay();                  //  Wait for Power to Setup

}  //  End Power


TogglePROG()                    //  Toggle the "PROG" Bit
{

  outp( PortAddr+2, 0x003 );    //  Strobe Low

  StrobeDelay();

  outp( PortAddr+2, 0x002 );    //  Strobe Back to Low

  StrobeDelay();

  outp( PortAddr+2, 0x004 );    //  Turn Off the '245 Drivers

  StrobeDelay();

}  //  End TogglePROG


Reset( int Value )              // Set the Correct Reset Value
{

  switch ( Value )
    {
      case 0:                   //  Turn OFF Reset
        outp( PortAddr, Value374 = Value374 & 0x09F );
        break;
      case 5:                   //  Reset to +5 Volts
        outp( PortAddr, Value374 = ( Value374 & 0x09F ) | 0x040 );
        break;
      case 12:                  //  Reset to +12 Volts
        outp( PortAddr, Value374 = ( Value374 & 0x09F ) | 0x020 );
        break;
    } /*  endswitch  */

  outp( PortAddr+2, 0x00D );
  outp( PortAddr+2, 0x004 );    //  Turning on Power, Strobe High

  StrobeDelay();

}  //  End Reset


ProgPin( int Value )            // Set the Correct Programming Pins
{                               //  Bit 0 of Value - P3.3 - PP Bit 4
                                //  Bit 1          - P3.4 - PP Bit 1
int  Temp = 0;                  //  Bit 2          - P3.5 - PP Bit 0
                                //  Bit 3          - P3.7 - PP Bit 2

  if (( Value & 1 ) != 0 )      //  Set the Bits to Write
    Temp = 0x010;
  if (( Value & 2 ) != 0 )
    Temp += 2;
  if (( Value & 4 ) != 0 )
    Temp += 1;
  if (( Value & 8 ) != 0 )
    Temp += 4;
  outp( PortAddr, Value374 = ( Value374 & 0x0E8 ) | Temp );

  outp( PortAddr+2, 0x00D );
  outp( PortAddr+2, 0x004 );    //  Turning on Power, Strobe High

  StrobeDelay();

}  //  End ProgPin


MemFree()                       //  Free the Memory Used by "Program"
{

union REGS regs;
struct SREGS sregs;


  regs.h.ah = 0x049;
  sregs.es = (long) Program / 0x010000;
  intdosx( &regs, &regs, &sregs );

}  /*  End MemFree  */


int cline( int argc,
  char *argv[],
  char *envp )
{

union REGS regs;

int  i, k;
long j;
int  handle;
int  retvalue = 2;              //  Return Value - Assume Good


  FileName[ 0 ] = '\0';         //  Mark the Start of the FileName

  if ( argc > 1 )               //  Only Handle if there are Args
    for ( i = 1; ( i < argc ) && ( retvalue != -1 ); i++ )
      switch ( argv[ i ][ 0 ])  //  Look at Each Character
        {
          case '1':             //  Handle the Port Specification
            if ( Port == -1 )
              {                 //  Specify the Port Value
                Port = 1;  PortAddr = 0x0378;
              }
            else
              {
                printf( "Multiple Port Specification\n" );
                printf( "Only ONE Port can be Defined\n" );
                retvalue = -1;
              }  /*  endif  */
            break;
          case '2':             //  Handle the Port Specification
            if ( Port == -1 )
              {                 //  Specify the Port Value
                Port = 2;  PortAddr = 0x0278;
              }
            else
              {
                printf( "Multiple Port Specification\n" );
                printf( "Only ONE Port can be Defined\n" );
                retvalue = -1;
              }  /*  endif  */
            break;
          default:;             //  Else, File Name Specified
            strncpy( &FileName[ 1 ], &argv[ i ][ 0 ],
                            FileName[ 0 ] = strlen( &argv[ i ][ 0 ]));
            for ( j = FileName[ 0 ] + 1; ( j > 1 ) &&
                                          ( FileName[ j ] != '\\' ) &&
                                      ( FileName[ j ] != '.' ); j-- );

            if ( FileName[ j ] != '.' )
              if ( FileName[ 0 ] > 59 )
                {
                  printf( "File name is TOO LARGE\n" );
                  retvalue = -1;
                }
              else
                {
                  strncpy( &FileName[ 1 + FileName[ 0 ]], ".hex", 4 );
                  FileName[ 0 ] += 4;
                }  /*  endif  */

            if ( retvalue != -1 )
              {
                FileName[ 1 + FileName[ 0 ]++ ] = '\0';
                if (( handle = open( &FileName[ 1 ],
                                        O_RDONLY | O_BINARY )) == -1 )
                  {
                    printf( "File \"%s\" was NOT FOUND\n",
                                                      &FileName[ 1 ]);
                    retvalue = -1;
                  }
                else            //  File was Opened, Okay, Read it
                  {
                    FileEnd = lseek( handle, 0L, SEEK_END );
                    if ( FileEnd == 0 )
                      {
                        printf( "File \"%s\" is EMPTY\n",
                                                      &FileName[ 1 ]);
                        retvalue = 1;
                      }
                    else        //  Get Memory & Store the .hex file
                      {

                        lseek( handle, 0L, SEEK_SET );

                        regs.h.ah = 0x048;
                        regs.x.bx = ( FileEnd + 0x010 )/ 0x010;
                        intdos( &regs, &regs );
                        (long) Program = regs.x.ax * 0x010000;

                        read( handle, Program, FileEnd );
                                //  Read in the .hex File
                        if ( Program[ FileEnd - 1 ] == 0x01A )
                          FileEnd--;
                        Program[ FileEnd++ ] = '\0';
                        if (( Program[ 0 ] != ':' ) ||
                                   ( strcmp( &Program[ FileEnd - 14 ],
                                           ":00000001FF\r\n" ) != 0 ))
                          {
                            printf( "File \"%s\" is NOT a Valid Hex File\n",
                                                      &FileName[ 1 ]);
                            retvalue = -1;
                            MemFree();  //  Free Up the Memory Used
                          }  /*  endif  */
                      }  /*  endif  */
                  }  /*  endif  */
              }  /*  endif  */
        } /*  endswitch  */

  if ( FileName[ 0 ] == '\0' )
    {
      printf( "NO Filename Specified\n" );
      retvalue = -1;
    }  /*  endif  */

  if ( Port == -1 )             //  No Port Specified
    {
      Port = 1;                 //  Default to "LPT1"
      PortAddr = 0x0378;
    }  /*  endif  */

  return( retvalue );

}  /*  End cline  */

int ReadHex( char far * Data )
{                               //  Convert two characters to Hex

int retvalue;


  if ( Data[ 0 ] >= 'A' )
    retvalue = ( Data[ 0 ] - 'A' + 10 ) * 0x010;
  else
    retvalue = ( Data[ 0 ] - '0' ) * 0x010;

  if ( Data[ 1 ] >= 'A' )
    retvalue += ( Data[ 1 ] - 'A' + 10 );
  else
    retvalue += ( Data[ 1 ] - '0' );

  return( retvalue );

}  //  End ReadHex


Prog()                          //  Now, Program the Part
{

int  retvalue = 0;
int  LineLength;                //  Number of Characters to Write To
int  LineAddress;

int  Temp;                      //  Data Read from the File
int  Data;                      //  Data to Program into the MCU

int  i = 0;
int  j, k;


  Power( 0 );                   //  Make Sure the Power is Off

  Power( 1 );                   //  Cycle the Power On

  Reset( 12 );

  printf( "\nErasing Flash\n" );

  ProgPin( 0x001 );             //  Erase the Flash

  outp( PortAddr + 2, 0x005 );  //  Pulse the Strobe Low
  TimeDelay();
  outp( PortAddr + 2, 0x004 );
  StrobeDelay();

  Reset( 5 );
  ProgPin( 0x00E );             //  Put Into Programming Mode

  printf( "\nProgramming" );

  while (( i++ < ( FileEnd - 14 )) && ( retvalue == 0 ))
    {                           //  Loop Through Each Line

      printf( "." );            //  Show the Progress

      LineLength = ReadHex( &Program[ i ]);  i += 2;
      LineAddress = ( ReadHex( &Program[ i ]) * 0x0100 ) +
                                          ReadHex( &Program[ i + 2 ]);
      i += 6;                   //  Skip Over Address/Line Good

      for ( ; CurAddress < LineAddress; )
        PCIncrement();          //  Increment to the New Address

      for ( j = 0; ( j < LineLength ) && ( retvalue == 0 ); j++ )
        {                       //  Now Put in the Data

          Reset( 12 );          //  Turn on 12 Volt Reset

          StrobeDelay();

          Temp = ReadHex( &Program[ i ]);  i += 2;

          SendData( Temp );     //  Output the Data to P1

          TogglePROG();         //  Strobe Out the Data

          TimeDelay();          //  #### - Not Polling Yet...

          Reset( 5 );           //  Reset Down to +5 Volts

          Data = ReadData() & 0x0FF;  //  Read the Data Back

          PCIncrement();        //  Increment the Address

          if ( Data != Temp ) { //  Miscompare on Write

            printf( "\nData Miscompare at Address 0x0%04X\n",
                                                     CurAddress - 1 );
            printf( "0x0%2X Read Back, 0x0%2X Expected\n",
                                         Data & 0x0FF, Temp & 0x0FF );
            retvalue = -1;
          }  //  endif
        }  /*  endfor  */

      i += 4;                   //  Skip Last 4 Characters on Line

    }  /*  endwhile  */

  Reset( 0 );                   //  Finished, Reset Off

  Power( 0 );

}  //  End Prog


//  Mainline
main(argc, argv, envp)
  int argc;
  char *argv[];
  char *envp;
{

int  retvalue;


  printf( "PROG35G\n" );
  printf( "Written by: Myke Predko\n\n" );

  if (( retvalue = cline( argc, argv, envp )) != -1 )
    {

      Prog();                   //  If File Name and Port Okay, Program

      MemFree();                //  Free the Memory Created by "cline"

    }  /*  endif  */

}  // End Mainline
