//  GetFont - Read the Font Information from 0x0F000:0x0FA6E
//
//  This program reads the Font information from the PC's ROM and
//   Stores it into a File that Can be included by an 8051 to get
//   Font information for shifting.
//
//  Myke Predko
//
//  Started: July 2, 1998
//  Updated:


//  Includes
#include <stdio.h>
#include <fcntl.h>              //  File Stuff Includes
#include <sys/types.h>
#include <sys/stat.h>
#include <io.h>
#include <string.h>


//  Defines


//  Global Variables
int  handle;                    //  DOS File Handle
char far * FontPointer;         //  Pointer to the Font Memory
char Line0[ 256 ];              //  Font Information
char Line1[ 256 ];
char Line2[ 256 ];
char Line3[ 256 ];
char Line4[ 256 ];
char Line5[ 256 ];
char Line6[ 256 ];
char Line7[ 256 ];


//  Subroutines
GetDec( int SaveChar, char far * Number )
{                               //  Save the Decimal Number

int  Temp;


  SaveChar &= 0x0FF;            //  Make Sure it's Valid Hex

  Number[ 0 ] = '0';
  Number[ 3 ] = 'h';

  Temp = SaveChar >> 4;         //  Do High Nybble
  if ( Temp > 9 )
    Number[ 1 ] = Temp + 'A' - 10;
  else
    Number[ 1 ] = Temp + '0';

  Temp = SaveChar & 0x00F;      //  Do High Nybble
  if ( Temp > 9 )
    Number[ 2 ] = Temp + 'A' - 10;
  else
    Number[ 2 ] = Temp + '0';

}  //  End GetDec


DoFont( char far * Line, int FontNum )  //  Load the Font Information
{                               //   into "Handle"

int  i;

char far * Number = "1234";     //  ASCIIZ String for File


  write( handle, "FontLine", 8 );
  Number[ 0 ] = FontNum + '0';
  write( handle, Number, 1 );
  write( handle, ":\r\n", 3 );

  write( handle, "  inc A\r\n", strlen( "  inc A\r\n" ));
  write( handle, "  jnz FontLine_Skip",
                                     strlen( "  jnz FontLine_Skip" ));
  write( handle, Number, 1 );
  write( handle, "\r\n", 2 );

  GetDec( Line[ 255 ], Number );
  write( handle, "  mov A,#", strlen( "  mov A,#" ));
  write( handle, Number, 4 );
  write( handle, "\r\n", 2 );

  write( handle, "  nop\r\n", strlen( "  nop\r\n" ));

  write( handle, "  ret\r\n", strlen( "  ret\r\n" ));

  write( handle, "FontLine_Skip", strlen( "FontLine_Skip" ));
  Number[ 0 ] = FontNum + '0';
  write( handle, Number, 1 );
  write( handle, ":\r\n", 3 );

  write( handle, "  movc A,@A+PC\r\n",strlen( "  movc A,@A+PC\r\n" ));

  write( handle, "  ret\r\n", strlen( "  ret\r\n" ));

  for( i = 0; i < 255; i++ ) {  //  Store the Table Information
                                //   On a Line by Line Basis
    write( handle, "  db  ", strlen( "  db  " ));
    GetDec( Line[ i ], Number );
    write( handle, Number, 4 );
    if ( i < 32 )
      {
        write( handle, "   ;  ", strlen( "   ;  " ));
        GetDec( i, Number );
        write( handle, Number, 4 );
      }
    else
      {
        write( handle, "   ;  \"", strlen( "   ;  \"" ));
        Number[ 0 ] = i;
        write( handle, Number, 1 );
        write( handle, "\"", 1 );
      }  /*  endif  */
    write( handle, "\r\n", 2 );

  }  //  endfor

}  //  End DoFont

//  Mainline
main()
{

int  i;


  printf( "GetFont - Read the 8x8 Font in the PC\'s ROM\n\n" );
  printf( "Myke Predko - July 2, 1998\n" );

  (long) FontPointer = 0x0F000FA6E;  //  Point to the Start of the
                                //   Font Memory

  for ( i = 0; i < 256; i++ ) { //  Read through the Font Memory

    Line0[ i ] = *FontPointer++;//  Store in Each of the Font Arrays
    Line1[ i ] = *FontPointer++;
    Line2[ i ] = *FontPointer++;
    Line3[ i ] = *FontPointer++;
    Line4[ i ] = *FontPointer++;
    Line5[ i ] = *FontPointer++;
    Line6[ i ] = *FontPointer++;
    Line7[ i ] = *FontPointer++;

  }

  handle = open( "font.inc", O_CREAT | O_BINARY | O_WRONLY,
                                                 S_IREAD | S_IWRITE );
  DoFont( Line0, 0 );           //  Store the Font Programs
  DoFont( Line1, 1 );
  DoFont( Line2, 2 );
  DoFont( Line3, 3 );
  DoFont( Line4, 4 );
  DoFont( Line5, 5 );
  DoFont( Line6, 6 );
  DoFont( Line7, 7 );

  close( handle );              //  Finished with the file

}  //  End main
